@@ -3,7 +3,7 @@
 //  celestia
 //
 //  Created by Bob Ippolito on Wed Jun 05 2002.
-//  Copyright (c) 2002 Chris Laurel. All rights reserved.
+//  Copyright (C) 2007, Celestia Development Team
 //
 
 #import "CelestiaAppCore.h"
@@ -14,21 +14,47 @@
 #import "CelestiaSelection_PrivateAPI.h"
 #import "CelestiaSimulation_PrivateAPI.h"
 #import "CelestiaRenderer_PrivateAPI.h"
-#import "CelestiaUniversalCoord_PrivateAPI.h"
-#import "CelestiaUniverse_PrivateAPI.h"
 #import "Astro.h"
 
 #import "CelestiaController.h"
 #include "celestiacore.h"
 #include "qtcapture.h"
+#import <Carbon/Carbon.h>
+
+class MacSettingsWatcher : public CelestiaWatcher
+{
+private:
+    CelestiaSettings *settings;
+
+public:
+    MacSettingsWatcher(CelestiaAppCore *_appCore,
+                       CelestiaSettings* _settings) :
+        CelestiaWatcher(*[_appCore appCore]), settings(_settings)
+    {};
+
+    void notifyChange(CelestiaCore *, int flags)
+    {
+        if ( 0 != (flags & (
+              CelestiaCore::LabelFlagsChanged
+            | CelestiaCore::RenderFlagsChanged
+            | CelestiaCore::VerbosityLevelChanged
+            | CelestiaCore::TimeZoneChanged
+            | CelestiaCore::AmbientLightChanged
+            | CelestiaCore::FaintestChanged
+            )) )
+        {
+            [settings validateItems];
+        }
+    };
+};
 
 class MacOSXAlerter : public CelestiaCore::Alerter
 {
 public:
     MacOSXAlerter() {};
-    ~MacOSXAlerter() {};
+    virtual ~MacOSXAlerter() {};
 
-    void fatalError(const std::string& msg)
+    virtual void fatalError(const std::string& msg)
     {
         NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
         NSLog(@"alerter fatalError!");
@@ -52,20 +78,62 @@ public:
     };
 };
 
+class MacOSXCursorHandler : public CelestiaCore::CursorHandler
+{
+public:
+    MacOSXCursorHandler() : cursor(kThemeArrowCursor),
+        shape(CelestiaCore::ArrowCursor) {};
+    virtual ~MacOSXCursorHandler() {};
+    virtual void setCursorShape(CelestiaCore::CursorShape aShape)
+    {
+        ThemeCursor changedCursor;
+
+        switch (aShape)
+        {
+        case CelestiaCore::SizeVerCursor:
+            changedCursor =
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
+            kThemeResizeUpDownCursor
+#else
+            kThemeClosedHandCursor
+#endif
+            ;
+            break;
+        case CelestiaCore::SizeHorCursor:
+            changedCursor =
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
+            kThemeResizeLeftRightCursor
+#else
+            kThemeClosedHandCursor
+#endif
+            ;
+            break;
+        default:
+            changedCursor = kThemeArrowCursor;
+            break;                
+        }
+
+        if (changedCursor != cursor)
+        {
+            SetThemeCursor(changedCursor);
+            cursor = changedCursor;
+            shape = aShape;
+        }
+    };
+
+    virtual CelestiaCore::CursorShape getCursorShape() const
+    {
+        return shape;
+    };
+    
+private:
+    ThemeCursor cursor;
+    CelestiaCore::CursorShape shape;
+};
+
 
 CelestiaAppCore *_sharedCelestiaAppCore;
 CelestiaCore *appCore;
-NSInvocation *contextMenuCallbackInvocation;
-
-void ContextMenuCallback(float x,float y, Selection selection) {
-    CelestiaSelection *csel;
-    NSPoint myPoint = NSMakePoint(x,y);
-    if (contextMenuCallbackInvocation == nil) return;
-    csel = [[[CelestiaSelection alloc] initWithSelection:selection] autorelease];
-    [contextMenuCallbackInvocation setArgument:&csel atIndex:2];
-    [contextMenuCallbackInvocation setArgument:&myPoint atIndex:3];
-    [contextMenuCallbackInvocation invoke];
-}
 
 @implementation CelestiaAppCore
 
@@ -205,14 +273,11 @@ void ContextMenuCallback(float x,float y, Selection selection) {
     return cModifiers;
 }
 
-static NSMutableDictionary* tagDict;
 
 +(void)initialize
 {
     _sharedCelestiaAppCore = nil;
-    contextMenuCallbackInvocation = nil;
     appCore = NULL;
-    tagDict = [[ NSMutableDictionary dictionaryWithCapacity: 100 ] retain];
 }
 
 +(CelestiaAppCore *)sharedAppCore
@@ -229,9 +294,7 @@ static NSMutableDictionary* tagDict;
         return _sharedCelestiaAppCore;
     }
     self = [super init];
-    appCore = new CelestiaCore();
-    appCore->setAlerter(new MacOSXAlerter());
-    contextMenuCallbackInvocation = NULL;
+    appCore = NULL;
     _destinations = nil;
     return self;
 }
@@ -248,20 +311,25 @@ static NSMutableDictionary* tagDict;
         [_destinations release];
         _destinations = nil;
     }
-    if (contextMenuCallbackInvocation != nil) {
-        [contextMenuCallbackInvocation release];
-        contextMenuCallbackInvocation = nil;
-    }
     if (appCore != NULL) {
+        // appCore doesn't own the custom alerter and cursor
+        // handler for some reason so we assume responsibility
+        // for cleanup
+        if (appCore->getAlerter())
+            delete appCore->getAlerter();
+        if (appCore->getCursorHandler())
+            delete appCore->getCursorHandler();
         delete appCore;
         appCore = NULL;
     }
     _sharedCelestiaAppCore = nil;
     [super dealloc];
 }
-    
+
 -(BOOL)initSimulation
 {
+    BOOL result = NO;
+    appCore = new CelestiaCore();
     NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
     NSString *confFileSetting;
     std::string confFile;
@@ -275,79 +343,6 @@ static NSMutableDictionary* tagDict;
         confFile = [confFileSetting stdString];
     }
 
-#ifndef NO_VP_WORKAROUND
-
-    std::string VP_PROBLEM_EXT      = "GL_ARB_vertex_program";
-    NSString    *VP_PATCH_SCRIPT    = @"vp_patch.sh";
-    NSString    *VP_PATCH_SHELL     = @"/bin/zsh";
-    NSString    *CELESTIA_CFG       = @"celestia.cfg";
-
-    const char *VP_PROBLEM_RENDERERS[] = { "ATI Radeon 9200" };
-    const char *glRenderer = (const char *) glGetString(GL_RENDERER);
-    BOOL shouldWorkaround = NO;
-    size_t i = 0;
-
-    if (glRenderer)
-    {
-        for (; i < (sizeof VP_PROBLEM_RENDERERS)/sizeof(char *); ++i)
-        {
-            if (strstr(glRenderer, VP_PROBLEM_RENDERERS[i]))
-            {
-                shouldWorkaround = YES;
-                break;
-            }
-        }
-    }
-
-    if (shouldWorkaround)
-    {
-        // Check if workaround is already installed
-        CelestiaConfig *cfg = ReadCelestiaConfig([CELESTIA_CFG stdString]);
-
-        if (cfg && std::find(cfg->ignoreGLExtensions.begin(), cfg->ignoreGLExtensions.end(), VP_PROBLEM_EXT) == cfg->ignoreGLExtensions.end())
-        {
-            delete cfg;
-            if (NSRunAlertPanel([NSString stringWithFormat: NSLocalizedString(@"It appears you are running Celestia on %s hardware. Do you wish to install a workaround?",nil), VP_PROBLEM_RENDERERS[i]],
-                                [NSString stringWithFormat: NSLocalizedString(@"A shell script will be run to modify your %@, adding an IgnoreGLExtensions directive. This can prevent freezing issues.",nil), CELESTIA_CFG],
-                                NSLocalizedString(@"Yes",nil),
-                                NSLocalizedString(@"No",nil),
-                                nil) == NSAlertDefaultReturn)
-            {
-                // Install it
-                NSFileManager *fm = [NSFileManager defaultManager];
-                NSString *cfgPath = [[fm currentDirectoryPath] stringByAppendingPathComponent: CELESTIA_CFG];
-                NSString *toolPath = [[NSBundle mainBundle] pathForResource: VP_PATCH_SCRIPT ofType: @""];
-                BOOL patchInstalled = NO;
-
-                if ([fm isWritableFileAtPath: cfgPath] && toolPath)
-                {
-                    NSArray *taskArgs = [NSArray arrayWithObjects:
-                        toolPath, cfgPath, nil];
-                    NSTask *theTask = [NSTask launchedTaskWithLaunchPath: VP_PATCH_SHELL
-                                                               arguments: taskArgs];
-                    if (theTask)
-                    {
-                        [theTask waitUntilExit];
-                        patchInstalled = ([theTask terminationStatus] == 0);
-                    }
-                }
-
-                if (patchInstalled)
-                {
-                    NSRunAlertPanel(NSLocalizedString(@"Workaround successfully installed.",nil),
-                                    [NSString stringWithFormat: NSLocalizedString(@"Your original %@ has been backed up.",nil), CELESTIA_CFG],
-                                    nil, nil, nil);
-                }
-                else
-                {
-                    [[CelestiaController shared] fatalError: NSLocalizedString(@"There was a problem installing the workaround. You may be running from a disk image, which is write-protected. Please try copying the CelestiaResources folder to your home directory as described in the README. You can also attempt to perform the workaround manually by following the instructions in the README.",nil)];
-                    [[CelestiaController shared] fatalError: nil];
-                }
-            }
-        }
-    }
-#endif NO_VP_WORKAROUND
-
     if ((extrasDirsSetting = [prefs stringArrayForKey:@"extrasDirs"]))
     {
         NSEnumerator *iter = [extrasDirsSetting objectEnumerator];
@@ -355,9 +350,18 @@ static NSMutableDictionary* tagDict;
             extrasDirs.push_back([extrasDir stdString]);
     }
 
-    return (BOOL)appCore->initSimulation(!confFile.empty() ? &confFile : nil,
-                                         extrasDirsSetting ? &extrasDirs : nil,
-                                         &progressNotifier);
+    appCore->setAlerter(new MacOSXAlerter());
+    appCore->setCursorHandler(new MacOSXCursorHandler());
+    result = appCore->initSimulation(!confFile.empty() ? &confFile : nil,
+                                     extrasDirsSetting ? &extrasDirs : nil,
+                                     &progressNotifier);
+    if (result)
+    {
+        CelestiaSettings *settings = [CelestiaSettings shared];
+        new MacSettingsWatcher(self, settings); // adds itself to the appCore
+    }
+
+    return result;
 }
 
 -(BOOL)initRenderer
@@ -365,10 +369,9 @@ static NSMutableDictionary* tagDict;
     return (BOOL)appCore->initRenderer();
 }
 
--(void)start:(NSDate *)date withTimeZone:(NSTimeZone *)timeZone
+-(void)start:(NSDate *)date
 {
     appCore->start([[Astro julianDate:date] doubleValue]);
-    [self setTimeZone:timeZone withDate:date];
 }
 
 -(void)charEntered:(char)c withModifiers:(int)modifiers
@@ -377,6 +380,11 @@ static NSMutableDictionary* tagDict;
     appCore->charEntered(c, modifiers);
 }
 
+-(void)charEntered:(NSString *)string
+{
+    appCore->charEntered([string UTF8String]);
+}
+
 -(void)keyDown:(int)c
 {
     appCore->keyDown(c);
@@ -412,6 +420,11 @@ static NSMutableDictionary* tagDict;
     appCore->mouseButtonUp(coord.x,coord.y,modifiers);
 }
 
+-(void)mouseMove:(NSPoint)coord
+{
+    appCore->mouseMove(coord.x,coord.y);
+}
+
 -(void)mouseMove:(NSPoint)delta modifiers:(int)modifiers
 {
     appCore->mouseMove(delta.x,delta.y,modifiers);
@@ -491,7 +504,7 @@ static NSMutableDictionary* tagDict;
 -(void)setTimeZone:(NSTimeZone *)timeZone withDate:(NSDate *)date
 {
     appCore->setTimeZoneBias([timeZone secondsFromGMTForDate:date]);
-    appCore->setTimeZoneName([[timeZone abbreviationForDate:date] cString]);
+    appCore->setTimeZoneName([[timeZone abbreviationForDate:date] stdString]);
 }
 
 -(int)textEnterMode
@@ -514,23 +527,6 @@ static NSMutableDictionary* tagDict;
     appCore->setHudDetail(hudDetail);
 }
 
--(void)setContextMenuCallback:(id)cObj
-{
-    SEL callbackSelector = @selector(contextMenuCallback:location:);
-    if (contextMenuCallbackInvocation != nil) {
-        [contextMenuCallbackInvocation release];
-        contextMenuCallbackInvocation = nil;
-    }
-    if (cObj == nil)
-        appCore->setContextMenuCallback(NULL);
-    else {
-        appCore->setContextMenuCallback(ContextMenuCallback);
-        contextMenuCallbackInvocation = [[NSInvocation invocationWithMethodSignature:[[cObj class] instanceMethodSignatureForSelector:callbackSelector]] retain];
-        [contextMenuCallbackInvocation setSelector:callbackSelector];
-        [contextMenuCallbackInvocation setTarget:cObj];
-    }
-}
-
 -(void)back
 {
     appCore->back();
@@ -544,12 +540,11 @@ static NSMutableDictionary* tagDict;
     appCore->forward();
 }
 
-- (NSString *) currentURL
+-(NSString *) currentURL
 {
-     Url currentUrl = Url(appCore, Url::Absolute);
-//   NSURL *url = [NSURL URLWithString: [ NSString stringWithStdString: currentUrl.getAsString() ]];
+    Url currentUrl = Url(appCore, Url::Absolute);
     NSString *url = [ NSString stringWithStdString: currentUrl.getAsString() ];
-   return url;
+    return url;
 }
 
 -(void)goToUrl:(NSString *)url
@@ -574,7 +569,7 @@ static NSMutableDictionary* tagDict;
 
 -(void)runScript:(NSString *)fileName
 {
-    appCore->runScript([fileName cString]);
+    appCore->runScript([fileName stdString]);
 }
 
 - (void) showInfoURL
@@ -632,7 +627,12 @@ static NSMutableDictionary* tagDict;
                 NSURL *theURL = [NSURL URLWithString: escUrl];
 
                 if (theURL != nil)
-                    [[NSWorkspace sharedWorkspace] openURL: theURL];
+                {
+                    if (nil == [theURL scheme])
+                        theURL = [NSURL fileURLWithPath: unescUrl];
+                    if (theURL)
+                        [[NSWorkspace sharedWorkspace] openURL: theURL];
+                }
             }
 
             [escUrl release];
@@ -640,12 +640,28 @@ static NSMutableDictionary* tagDict;
     }
 }
 
+-(unsigned int)aaSamples
+{
+    return appCore->getConfig()->aaSamples;
+}
+
+-(BOOL)glExtensionIgnored:(NSString *)extension
+{
+    CelestiaConfig *cfg = appCore->getConfig();
+    return std::find(cfg->ignoreGLExtensions.begin(), cfg->ignoreGLExtensions.end(), [extension stdString]) != cfg->ignoreGLExtensions.end();
+}
+
+-(void)setGLExtensionIgnored:(NSString *)extension
+{
+    appCore->getConfig()->ignoreGLExtensions.push_back([extension stdString]);
+}
+
 - (BOOL) captureMovie: (NSString*) filename width: (int) width height: (int) height
                               frameRate: (float) framerate
 {
     MovieCapture* movieCapture = new QTCapture();
 
-    bool success = movieCapture->start([filename cString], width, height, framerate);
+    bool success = movieCapture->start([filename stdString], width, height, framerate);
     if (success)
         appCore->initMovieCapture(movieCapture);
     else
@@ -655,4 +671,3 @@ static NSMutableDictionary* tagDict;
 }
 
 @end
-
