@@ -3,7 +3,7 @@
 //  celestia
 //
 //  Created by Bob Ippolito on Tue May 28 2002.
-//  Copyright (c) 2002 Chris Laurel. All rights reserved.
+//  Copyright (C) 2007, Celestia Development Team
 //
 
 #include <unistd.h>
@@ -13,7 +13,10 @@
 #import "FullScreenWindow.h"
 #import "SplashScreen.h"
 #import "SplashWindowController.h"
+#import "EclipseFinderController.h"
+#import "ScriptsController.h"
 #import <Carbon/Carbon.h>
+#import <OpenGL/gl.h>
 #import "CGLInfo.h"
 
 #include <float.h>
@@ -41,14 +44,21 @@ NSString* fatalErrorMessage;
     }
 
     if (firstInstance == nil ) firstInstance = self;
+    int cpuCount = 0;
+    size_t cpuCountSize = sizeof cpuCount;
+    if (0 == sysctlbyname("hw.ncpu", &cpuCount, &cpuCountSize, NULL, 0))
+    {
+        threaded = (cpuCount > 1);
+    }
     ready = NO;
     isDirty = YES;
     isFullScreen = NO;
     appCore = nil;
     fatalErrorMessage = nil;
     lastScript = nil;
-	
+
     [self setupResourceDirectory];
+    [scriptsController buildScriptMenu];
 
     //  hide main window until ready
     [[glView window] setAlphaValue: 0.0f];  //  not  [[glView window] orderOut: nil];
@@ -65,16 +75,18 @@ NSString* fatalErrorMessage;
     
     startupCondition = [[NSConditionLock alloc] initWithCondition: 0];
 
-    // start initialization thread
-#ifdef USE_THREADEDLOAD
-    [NSThread detachNewThreadSelector: @selector(startInitialization) toTarget: self
-    withObject: nil];
-
-    // wait for completion
-    [self performSelectorOnMainThread: @selector(waitWhileLoading:) withObject: nil waitUntilDone: NO ];
-#else
-    [self performSelector:@selector(startInitialization) withObject:nil afterDelay:0];
-#endif
+    if (threaded)
+    {
+        // start initialization thread
+        [NSThread detachNewThreadSelector: @selector(startInitialization) toTarget: self
+        withObject: nil];
+        // wait for completion
+        [self performSelectorOnMainThread: @selector(waitWhileLoading:) withObject: nil waitUntilDone: NO ];
+    }
+    else
+    {
+        [self performSelector:@selector(startInitialization) withObject:nil afterDelay:0];
+    }
 }
 
 
@@ -120,7 +132,7 @@ NSString* fatalErrorMessage;
         {
             if (![fileManager fileExistsAtPath: path = [[ mainBundle resourcePath ] stringByAppendingPathComponent: CELESTIA_RESOURCES_FOLDER ] isDirectory: &isFolder] || !isFolder)
             {
-                [self fatalError: @"It appears that the \"CelestiaResources\" directory has not been properly installed in the correct location as indicated in the installation instructions. \n\nPlease correct this and try again."];
+                [self fatalError: NSLocalizedString(@"It appears that the \"CelestiaResources\" directory has not been properly installed in the correct location as indicated in the installation instructions. \n\nPlease correct this and try again.",@"")];
                 [self fatalError: nil];
             }
         }
@@ -132,13 +144,10 @@ NSString* fatalErrorMessage;
 
 - (void)startInitialization
 {
-#ifdef DEBUG
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
-#endif
-#ifndef USE_THREADEDLOAD
-    [splashWindowController showWindow];
-#endif
-    // initialize simulator in separate thread to allow loading indicator window while waiting
+
+    if (!threaded) [splashWindowController showWindow];
+    [[glView openGLContext] makeCurrentContext];
 #ifdef DEBUG
     NSDate *t = [NSDate date];
 #endif
@@ -149,12 +158,11 @@ NSString* fatalErrorMessage;
 #ifdef DEBUG
         [pool release];
 #endif
-        [self fatalError: @"Error loading data files. Celestia will now quit."];
-#ifndef USE_THREADEDLOAD
-        [self fatalError: nil];
-#else
-        [NSThread exit];
-#endif
+        [self fatalError: NSLocalizedString(@"Error loading data files. Celestia will now quit.",@"")];
+        if (!threaded)
+            [self fatalError: nil];
+        else
+            [NSThread exit];
         return;
     }
 
@@ -163,16 +171,15 @@ NSString* fatalErrorMessage;
 #endif
     [startupCondition lock];
     [startupCondition unlockWithCondition: 1];
-#ifndef USE_THREADEDLOAD
-    [splashWindowController close];
-    // complete startup
-    [self fatalError: nil];
-    [self finishInitialization];
-#endif
+    if (!threaded)
+    {
+        [splashWindowController close];
+        // complete startup
+        [self fatalError: nil];
+        [self finishInitialization];
+    }
 
-#ifdef DEBUG
     [pool release];
-#endif
 }
 
 
@@ -183,7 +190,7 @@ NSString* fatalErrorMessage;
     {
         if (fatalErrorMessage == nil) return;
         [splashWindowController close];
-        NSRunAlertPanel(@"Fatal Error", fatalErrorMessage, nil, nil, nil);
+        NSRunAlertPanel(NSLocalizedString(@"Fatal Error",@""), fatalErrorMessage, nil, nil, nil);
         fatalErrorMessage = nil;    // user could cancel the terminate
         [NSApp terminate:self];
         return;
@@ -194,17 +201,17 @@ NSString* fatalErrorMessage;
 - (void) waitWhileLoading: (id) obj
 {
     // display loading indicator window while loading
-    
+
     static NSModalSession session = nil;
 
     if ( [startupCondition condition] == 0 ) 
     {
         if ( session != nil )
             return;
-//        [loadingIndicator startAnimation: nil];
-        // begin modal session for loading panel
+        // beginModalSession also displays the window, but the centering
+        // is wrong so do the display and centering beforehand
+        [splashWindowController showWindow];
         session = [NSApp beginModalSessionForWindow: [splashWindowController window]];
-        // modal session runloop
         for (;;) 
         {
             if ( fatalErrorMessage != nil )
@@ -214,7 +221,6 @@ NSString* fatalErrorMessage;
             if ( [startupCondition condition] != 0 )
                 break;
         }
-        // terminate modal session for loading panel
         [NSApp endModalSession:session];
     }
     // check for fatal error in loading thread
@@ -234,19 +240,7 @@ NSString* fatalErrorMessage;
     [[CelestiaFavorites sharedFavorites] setSynchronize:menuCallback];
     [[CelestiaFavorites sharedFavorites] synchronize];
 }
-/*
--(void) setupRenderPanel
-{
-//    [[renderPanelController window] setAlphaValue: 0.8f];
-//    [[renderPanelController window] setLevel: NSFloatingWindowLevel];
-//    [[glView window] addChildWindow: [renderPanelController window] ordered: NSWindowAbove];
-    [[renderPanelController window] setMovableByWindowBackground: YES];
-    [[renderPanelController window] setHidesOnDeactivate: YES];
-    [[renderPanelController window] setReleasedWhenClosed: NO];
-    [[renderPanelController window] setOneShot: NO];
-//    [renderPanelController finishSetup];
-}
-*/
+
 -(void) startGLView
 {
     [[glView window] setAutodisplay:YES];
@@ -254,7 +248,8 @@ NSString* fatalErrorMessage;
     [[glView window] setFrameUsingName: @"Celestia"];
     [[glView window] setAlphaValue: 1.0f];
     [[glView window] setFrameAutosaveName: @"Celestia"];
-    [[glView window] makeMainWindow ];
+    if ([[glView window] canBecomeMainWindow])
+        [[glView window] makeMainWindow ];
     [[glView window] makeFirstResponder: glView ];
     [[glView window] makeKeyAndOrderFront: glView ];
     [glView registerForDraggedTypes:
@@ -262,8 +257,93 @@ NSString* fatalErrorMessage;
     [glView setNeedsDisplay:YES];
 }
 
+- (void)startRender
+{
+    [self startGLView];
+
+    if (isFullScreen)
+    {
+        isFullScreen = NO;
+        [self toggleFullScreen: self];
+    }
+
+    // workaround for fov problem
+    if (pendingUrl) [appCore goToUrl: pendingUrl];
+
+    if ([startupCondition tryLockWhenCondition: 1])
+        [startupCondition unlockWithCondition:  2];
+}
+
 - (void)finishInitialization
 {
+#ifndef NO_VP_WORKAROUND
+    NSString *VP_PROBLEM_EXT  = @"GL_ARB_vertex_program";
+    NSString *VP_PATCH_SCRIPT = @"vp_patch.sh";
+    NSString *VP_PATCH_SHELL  = @"/bin/zsh";
+    NSString *CELESTIA_CFG    = @"celestia.cfg";
+
+    const char *VP_PROBLEM_RENDERERS[] = { "ATI Radeon 9200" };
+    const char *glRenderer = (const char *) glGetString(GL_RENDERER);
+    BOOL shouldWorkaround = NO;
+    size_t i = 0;
+
+    if (glRenderer)
+    {
+        for (; i < (sizeof VP_PROBLEM_RENDERERS)/sizeof(char *); ++i)
+        {
+            if (strstr(glRenderer, VP_PROBLEM_RENDERERS[i]))
+            {
+                shouldWorkaround = YES;
+                break;
+            }
+        }
+    }
+
+    if (shouldWorkaround && ![appCore glExtensionIgnored: VP_PROBLEM_EXT])
+    {
+        if (NSRunAlertPanel([NSString stringWithFormat: NSLocalizedString(@"It appears you are running Celestia on %s hardware. Do you wish to install a workaround?",nil), VP_PROBLEM_RENDERERS[i]],
+                            [NSString stringWithFormat: NSLocalizedString(@"A shell script will be run to modify your %@, adding an IgnoreGLExtensions directive. This can prevent freezing issues.",nil), CELESTIA_CFG],
+                            NSLocalizedString(@"Yes",nil),
+                            NSLocalizedString(@"No",nil),
+                            nil) == NSAlertDefaultReturn)
+        {
+            // Install it
+            NSFileManager *fm = [NSFileManager defaultManager];
+            NSString *cfgPath = [[fm currentDirectoryPath] stringByAppendingPathComponent: CELESTIA_CFG];
+            NSString *toolPath = [[NSBundle mainBundle] pathForResource: VP_PATCH_SCRIPT ofType: @""];
+            BOOL patchInstalled = NO;
+
+            if ([fm isWritableFileAtPath: cfgPath] && toolPath)
+            {
+                NSArray *taskArgs = [NSArray arrayWithObjects:
+                    toolPath, cfgPath, nil];
+                NSTask *theTask = [NSTask launchedTaskWithLaunchPath: VP_PATCH_SHELL
+                                                           arguments: taskArgs];
+                if (theTask)
+                {
+                    [theTask waitUntilExit];
+                    patchInstalled = ([theTask terminationStatus] == 0);
+                }
+            }
+
+            if (patchInstalled)
+            {
+                // Have to apply same patch to config already loaded in memory
+                [appCore setGLExtensionIgnored: VP_PROBLEM_EXT];
+                NSRunAlertPanel(NSLocalizedString(@"Workaround successfully installed.",nil),
+                                [NSString stringWithFormat: NSLocalizedString(@"Your original %@ has been backed up.",nil), CELESTIA_CFG],
+                                nil, nil, nil);
+            }
+            else
+            {
+                [[CelestiaController shared] fatalError: NSLocalizedString(@"There was a problem installing the workaround. You may be running from a disk image, which is write-protected. Please try copying the CelestiaResources folder to your home directory as described in the README. You can also attempt to perform the workaround manually by following the instructions in the README.",nil)];
+                [[CelestiaController shared] fatalError: nil];
+            }
+        }
+    }
+#endif NO_VP_WORKAROUND
+
+    [glView setAASamples: [appCore aaSamples]];
     [appCore initRenderer];
 
     [self setupFavorites];
@@ -271,41 +351,36 @@ NSString* fatalErrorMessage;
     settings = [CelestiaSettings shared];
     [settings setControl: self];
     [settings scanForKeys: [renderPanelController window]];
-//    [self setupRenderPanel];
     [settings validateItems];
 
-    // load settings
-    [settings loadUserDefaults];
-    // Have to delay going full screen until after view is started
-    BOOL shouldGoFullScreen = isFullScreen;
-    isFullScreen = NO;
-
     // paste URL if pending
     if (pendingUrl != nil )
     {
         [ appCore setStartURL: pendingUrl ];
     }
 
-    // set the simulation starting time to the current system time
-    [appCore start:[NSDate date] withTimeZone:[NSTimeZone defaultTimeZone]];
+    [appCore start:[NSDate date]];
 
+    // load settings
+    // Delay applying settings until after setting simulation time
+    // This allows the timezone to be set correctly
+    [settings loadUserDefaults];
+    
     ready = YES;
     timer = [[NSTimer timerWithTimeInterval: 0.01 target: self selector:@selector(timeDisplay) userInfo:nil repeats:YES] retain];
     [[NSRunLoop currentRunLoop] addTimer:timer forMode:NSDefaultRunLoopMode];
 
-    [self startGLView];
+    // Threaded startup can allow app to be hidden during startup
+    // When this happens delay rendering until first unhide
+    // to prevent all sorts of problems
+    if (![NSApp isHidden])
+        [self startRender];
 
-    if (shouldGoFullScreen)
-        [self toggleFullScreen: self];
-
-    // run script if pending
+    // run script if pending (scripts can run without rendering)
     if (pendingScript != nil )
     {
         [self runScript: pendingScript ];
-    }
-
-    // workaround for fov problem
-    if (pendingUrl) [appCore goToUrl: pendingUrl];
+    }    
 }
 
 // Application Event Handler Methods ----------------------------------------------------------
@@ -330,32 +405,6 @@ NSString* fatalErrorMessage;
     else
         pendingUrl = [[[event descriptorAtIndex:1] stringValue] retain];
 }
-/*
-- (void) applicationDidBecomeActive:(NSNotification *) aNotification
-{
-    if (aNotification && ([aNotification object] == NSApp))
-    {
-        [self unpauseFullScreen];
-    }
-}
-*/
-- (void) applicationWillHide:(NSNotification *) aNotification
-{
-    if (aNotification && ([aNotification object] == NSApp))
-    {
-        [self pauseFullScreen];
-    }
-}
-
-- (void) applicationWillResignActive:(NSNotification *) aNotification
-{
-    if (isFullScreen && aNotification && ([aNotification object] == NSApp) && ![[aNotification object] isHidden])
-    {
-        // Hiding also causes deactivation - handle hiding separately
-        [self pauseFullScreen];
-        [[self window] orderBack: self];
-    }
-}
 
 /* On a multi-screen setup, user is able to change the resolution of the screen running Celestia from a different screen, or the menu bar position so handle that */
 - (void)applicationDidChangeScreenParameters:(NSNotification *) aNotification
@@ -364,7 +413,7 @@ NSString* fatalErrorMessage;
     {
         // If menu bar not on same screen, don't hide it anymore
         if (![self hideMenuBarOnActiveScreen])
-            ShowMenuBar();
+            SetSystemUIMode(kUIModeNormal, 0);
 
         NSScreen *screen = [[self window] screen];
         NSRect screenRect = [screen frame];
@@ -374,16 +423,37 @@ NSString* fatalErrorMessage;
     }
 }
 
+- (void)applicationDidHide:(NSNotification *)aNotification
+{
+    ready = NO;
+}
+
+- (void)applicationWillUnhide:(NSNotification *)aNotification
+{
+    if ( [startupCondition condition] == 0 ) return;
+    ready = YES;
+}
+
+- (void)applicationDidUnhide:(NSNotification *)aNotification
+{
+    if ( [startupCondition tryLockWhenCondition: 1] )
+    {
+        [startupCondition unlock];
+        [self startRender];
+    }
+}
+
 -(BOOL)applicationShouldTerminate:(id)sender
 {
-    [self pauseFullScreen];   // allow dialog to show
-
-   if (  NSRunAlertPanel(@"Quit Celestia?",@"Are you sure you want to quit Celestia?",@"Quit",@"Cancel",nil) != NSAlertDefaultReturn ) 
+   if (  NSRunAlertPanel(NSLocalizedString(@"Quit Celestia?",@""),
+                         NSLocalizedString(@"Are you sure you want to quit Celestia?",@""),
+                         NSLocalizedString(@"Quit",@""),
+                         NSLocalizedString(@"Cancel",@""),
+                         nil) != NSAlertDefaultReturn ) 
    {
-       [self unpauseFullScreen];
        return NO;
    }
-    
+
     if (timer != nil) {
         [timer invalidate];
         [timer release];
@@ -398,6 +468,7 @@ NSString* fatalErrorMessage;
     [settings storeUserDefaults];
 
     [lastScript release];
+    [eclipseFinderController release];
     [browserWindowController release];
     [helpWindowController release];
 
@@ -410,27 +481,6 @@ NSString* fatalErrorMessage;
 
 // Window Event Handler Methods ----------------------------------------------------------
 
-- (void)windowDidBecomeKey:(NSNotification *)aNotification
-{
-    [self unpauseFullScreen];
-}
-
-- (void)windowDidResignKey:(NSNotification *)aNotification
-{
-    NSNumber *mainScreenNumber =
-    [[[[self window] screen] deviceDescription] objectForKey:@"NSScreenNumber"];
-    NSScreen *otherScreen = [[NSApp keyWindow] screen];
-    NSNumber *otherScreenNumber = otherScreen ? 
-        [[otherScreen deviceDescription] objectForKey:@"NSScreenNumber"] :
-        nil;
-    
-    if (mainScreenNumber && otherScreenNumber &&
-        [mainScreenNumber isEqualTo:otherScreenNumber])
-    {
-        [self pauseFullScreen];
-    }
-}
-
 -(BOOL)windowShouldClose:(id)sender
 {
     [NSApp terminate:nil];
@@ -466,7 +516,7 @@ NSString* fatalErrorMessage;
                keyTime --;
        }    
 }
- 
+
 // Display Update Management Methods ----------------------------------------------------------
 
 - (void)setDirty
@@ -569,8 +619,8 @@ NSString* fatalErrorMessage;
             // Can't switch back to windowed mode, but hide full screen window
             // so user can still quit the program
             [[self window] orderOut: self];
-            ShowMenuBar();
-            [self fatalError: @"Unable to properly exit full screen mode. Celestia will now quit."];
+            SetSystemUIMode(kUIModeNormal, 0);
+            [self fatalError: NSLocalizedString(@"Unable to properly exit full screen mode. Celestia will now quit.",@"")];
             [self performSelector:@selector(fatalError:) withObject:nil afterDelay:0.1];
             return;
         }
@@ -579,7 +629,7 @@ NSString* fatalErrorMessage;
         [[glView openGLContext] setView: windowedView];
 
         [[self window] close];  // full screen window releases on close
-        ShowMenuBar();
+        SetSystemUIMode(kUIModeNormal, 0);
         [self setWindow: origWindow];
         glView = windowedView;
         [self setDirty];
@@ -596,10 +646,10 @@ NSString* fatalErrorMessage;
     [fullScreenView setMenu: [glView menu]];    // context menu
 
     FullScreenWindow *fullScreenWindow = [[FullScreenWindow alloc] initWithScreen: screen];
+    [fullScreenWindow fadeOutScreen];
 
     [fullScreenWindow setBackgroundColor: [NSColor blackColor]];
     [fullScreenWindow setReleasedWhenClosed: YES];
-    [fullScreenWindow setLevel: NSStatusWindowLevel];
     [self setWindow: fullScreenWindow]; // retains it
     [fullScreenWindow release];
     [fullScreenWindow setDelegate: self];
@@ -623,31 +673,11 @@ NSString* fatalErrorMessage;
     [glView update];
     [glView display];
 
-    CGDisplayRestoreColorSyncSettings();
     [fullScreenWindow makeMainWindow];
+    [fullScreenWindow restoreScreen];
     isFullScreen = YES;
 }
 
-/* Lowers the level of a full-screen window (to allow Cmd-Tabbing, etc) */
-- (void) pauseFullScreen
-{
-    if (isFullScreen)
-    {
-        [[self window] setLevel: NSNormalWindowLevel];
-        ShowMenuBar();
-    }
-}
-
-/* Resumes full screen after a pauseFullScreen */
-- (void) unpauseFullScreen
-{
-    if (isFullScreen)
-    {
-        [self hideMenuBarOnActiveScreen];
-        [[self window] setLevel: NSStatusWindowLevel];
-    }
-}
-
 - (BOOL) hideMenuBarOnActiveScreen
 {
     NSScreen *screen = [[self window] screen];
@@ -655,7 +685,7 @@ NSString* fatalErrorMessage;
     if (allScreens && [allScreens objectAtIndex: 0]!=screen)
         return NO;
 
-    HideMenuBar();
+    SetSystemUIMode(kUIModeAllHidden, kUIOptionAutoShowMenuBar);
     return YES;
 }
 
@@ -667,30 +697,17 @@ NSString* fatalErrorMessage;
     [appCore runScript: lastScript];       
 }
 
-- (void) openPanelDidEnd:(NSOpenPanel*)openPanel returnCode: (int) rc contextInfo: (void *) ci
-{
-    if (rc == NSOKButton )
-    {
-        NSString *path;
-        path = [openPanel filename];
-        [self runScript: path];       
-    }
-}
-
 - (IBAction) openScript: (id) sender
 {
     NSOpenPanel* panel = [NSOpenPanel openPanel];
     NSDocumentController *dc = [NSDocumentController sharedDocumentController];
-//    if (!lastScript) lastScript = [[NSHomeDirectory() stringByAppendingString: @"/"]  retain];
-
-    [ panel beginSheetForDirectory: nil //[lastScript stringByDeletingLastPathComponent]
-                              file: nil//[lastScript lastPathComponent]
-                             types: [dc fileExtensionsFromType:@"Celestia Script"]//[NSArray arrayWithObjects: @"cel", @"celx", nil ]
-                    modalForWindow: [glView window]
-                     modalDelegate: self
-                    didEndSelector: @selector(openPanelDidEnd:returnCode:contextInfo:)
-                       contextInfo: nil
-    ];
+    int result = [panel runModalForTypes: [dc fileExtensionsFromType:@"Celestia Script"]];
+    if (result == NSOKButton)
+    {
+        NSString *path;
+        path = [panel filename];
+        [self runScript: path];       
+    }
 }
 
 - (IBAction) rerunScript: (id) sender
@@ -708,7 +725,17 @@ NSString* fatalErrorMessage;
     [appCore forward];
 }
 
--(IBAction)showGLInfo:(id)sender
+- (IBAction) selectSatellite:(id)sender
+{
+    if (sender &&
+        [sender respondsToSelector: @selector(representedObject)] &&
+        [sender representedObject])
+    {
+        [[appCore simulation] setSelection: [[[CelestiaSelection alloc] initWithCelestiaBody: [sender representedObject]] autorelease]];
+    }
+}
+
+- (IBAction) showGLInfo:(id)sender
 {
     if (![glInfoPanel isVisible])
     {
@@ -741,13 +768,13 @@ NSString* fatalErrorMessage;
 - (IBAction) captureMovie: (id) sender
 {
 //  Remove following line to enable movie capture...
-	NSRunAlertPanel(@"No Movie Capture",@"Movie capture is not available in this version of Celestia.",nil,nil,nil); return;
+	NSRunAlertPanel(NSLocalizedString(@"No Movie Capture",@""), NSLocalizedString(@"Movie capture is not available in this version of Celestia.",@""),nil,nil,nil); return;
 
     NSSavePanel* panel = [NSSavePanel savePanel];
 	NSString* lastMovie = nil; // temporary; should be saved in defaults
 
 	[panel setRequiredFileType: @"mov"];
-	[panel setTitle: @"Capture Movie"];
+	[panel setTitle: NSLocalizedString(@"Capture Movie",@"")];
     [ panel beginSheetForDirectory:  [lastMovie stringByDeletingLastPathComponent]
                               file: [lastMovie lastPathComponent]
                     modalForWindow: [glView window]
@@ -759,7 +786,7 @@ NSString* fatalErrorMessage;
 
 // GUI Tag Methods ----------------------------------------------------------
 
-- (BOOL)     validateMenuItem: (id) item
+- (BOOL) validateMenuItem: (id) item
 {
     if ( [startupCondition condition] == 0 ) return NO;
     if ( [item action] == nil  ) return NO;
@@ -791,12 +818,6 @@ NSString* fatalErrorMessage;
     }
 }
 
-- (void) addSurfaceMenu: (NSMenu*) contextMenu
-{
-    [settings addSurfaceMenu: contextMenu ];
-}
-
-
 -(IBAction) showPanel: (id) sender
 {
     switch( [sender tag] )
@@ -805,6 +826,10 @@ NSString* fatalErrorMessage;
             if (!browserWindowController) browserWindowController = [[BrowserWindowController alloc] init];
             [browserWindowController showWindow: self];
             break;
+        case 1:
+            if (!eclipseFinderController) eclipseFinderController = [[EclipseFinderController alloc] init];
+            [eclipseFinderController showWindow: self];
+            break;
     }
 }
 
@@ -817,3 +842,24 @@ NSString* fatalErrorMessage;
 }
 
 @end
+
+#pragma mark -
+
+// Solution for keyDown sent but keyUp not sent for Cmd key combos.
+// Fixes the infamous Cmd+arrow "infinite spin"
+@interface CelestiaApplication : NSApplication
+@end
+
+@implementation CelestiaApplication
+- (void)sendEvent: (NSEvent *)aEvent
+{
+    if ([aEvent type] == NSKeyUp)
+    {
+        [[[self mainWindow] firstResponder] tryToPerform: @selector(keyUp:)
+                                                    with: aEvent];
+        return;
+    }
+    
+    [super sendEvent: aEvent];
+}
+@end
