@@ -17,6 +17,12 @@
 #include "astro.h"
 #include <celutil/util.h>
 
+#ifndef TARGET_OS_MAC
+#ifdef __GNUC__
+#include <langinfo.h>
+#endif
+#endif
+
 using namespace std;
 
 const double astro::speedOfLight = 299792.458; // km/s
@@ -376,7 +382,6 @@ double astro::meanEclipticObliquity(double jd)
     return J2000Obliquity - de;
 }
 
-
 astro::Date::Date()
 {
     year = 0;
@@ -407,6 +412,7 @@ astro::Date::Date(double jd)
 {
     int64 a = (int64) floor(jd + 0.5);
     wday = (a + 1) % 7;
+    if (wday < 0) wday += 7;
     double c;
     if (a < 2299161)
     {
@@ -441,6 +447,14 @@ astro::Date::Date(double jd)
     tzname = "UTC";
 }
 
+#ifndef TARGET_OS_MAC
+#ifdef __GNUC__	
+bool astro::Date::utf8Locale = false;
+bool astro::Date::utf8LocaleSet = false;
+iconv_t astro::Date::utf8Iconv;
+#endif
+#endif
+
 const char* astro::Date::toCStr(Format format) const
 {
     static char date[255];
@@ -472,7 +486,31 @@ const char* astro::Date::toCStr(Format format) const
         break;
     }
 
-    strftime(date, sizeof(date), strftime_format, &cal_time);
+    size_t in = strftime(date, sizeof(date), strftime_format, &cal_time);
+
+#ifndef TARGET_OS_MAC
+    // The string returned by strftime is in the current locale's charset
+    // which may not be UTF-8
+    if (!astro::Date::utf8LocaleSet)
+    {
+        // Checks the current locale's charset and if needed sets up
+        // an iconv handle
+        astro::Date::utf8Locale = !strcmp(nl_langinfo(CODESET), "UTF-8");
+        if (!astro::Date::utf8Locale) 
+            astro::Date::utf8Iconv = iconv_open("UTF-8", nl_langinfo(CODESET));
+        astro::Date::utf8LocaleSet = true;
+    }
+
+    if (!astro::Date::utf8Locale)
+    {
+        static char utf8Date[255];
+        char* outBuff = utf8Date;
+        char* inBuff = date;
+        size_t out = 255;
+        iconv(astro::Date::utf8Iconv, &inBuff, &in, &outBuff, &out);
+        return utf8Date;
+    }
+#endif
 #else
     switch(format)
     {
