@@ -16,6 +16,12 @@
 #include <celmath/vecmath.h>
 #include <celengine/univcoord.h>
 
+#ifndef TARGET_OS_MAC
+#ifdef __GNUC__
+#include <iconv.h>
+#endif
+#endif
+
 #define SOLAR_ABSMAG  4.83f
 #define LN_MAG        1.085736
 #define LY_PER_PARSEC 3.26167
@@ -56,6 +62,16 @@ namespace astro
         int utc_offset;     // offset from utc in seconds
         std::string tzname; // timezone name
         double seconds;
+
+#ifndef TARGET_OS_MAC
+#ifdef __GNUC__
+    private:
+        // Static members used to store locale related UTF-8 information
+        static bool utf8Locale;
+        static bool utf8LocaleSet;
+        static iconv_t utf8Iconv;
+#endif
+#endif
     };
 
     bool parseDate(const std::string&, Date&);
