@@ -1,6 +1,6 @@
 // body.h
 //
-// Copyright (C) 2001-2004 Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2001-2006 Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -13,6 +13,7 @@
 #include <string>
 #include <vector>
 #include <map>
+#include <celutil/utf8.h>
 #include <celmath/quaternion.h>
 #include <celengine/surface.h>
 #include <celengine/atmosphere.h>
@@ -21,7 +22,7 @@
 #include <celengine/location.h>
 #include <celengine/rotation.h>
 
-
+class ReferenceFrame;
 class Body;
 
 class PlanetarySystem
@@ -47,13 +48,18 @@ class PlanetarySystem
     typedef bool (*TraversalFunc)(Body*, void*);
 
     bool traverse(TraversalFunc, void*) const;
-    Body* find(std::string, bool deepSearch = false, bool i18n = false) const;
+    Body* find(const std::string&, bool deepSearch = false, bool i18n = false) const;
     std::vector<std::string> getCompletion(const std::string& _name, bool rec = true) const;
 
+ private:
+    typedef std::map<std::string, Body*, UTF8StringOrderingPredicate> ObjectIndex;
+
  private:
     Star* star;
     Body* primary;
     std::vector<Body*> satellites;
+    ObjectIndex objectIndex;  // index of bodies by name
+    ObjectIndex i18nObjectIndex;
 };
 
 
@@ -85,13 +91,17 @@ class Body
 
     enum
     {
-        Planet     = 0x01,
-        Moon       = 0x02,
-        Asteroid   = 0x04,
-        Comet      = 0x08,
-        Spacecraft = 0x10,
-        Invisible  = 0x20,
-        Unknown    = 0x10000,
+        Planet      =    0x01,
+        Moon        =    0x02,
+        Asteroid    =    0x04,
+        Comet       =    0x08,
+        Spacecraft  =    0x10,
+        Invisible   =    0x20,
+        Barycenter  =    0x40,
+        SmallBody   =    0x80,
+        DwarfPlanet =   0x100,
+        Stellar     =   0x200, // only used for orbit mask
+        Unknown     = 0x10000,
     };
 
     PlanetarySystem* getSystem() const;
@@ -99,10 +109,16 @@ class Body
     void setName(const std::string);
     Orbit* getOrbit() const;
     void setOrbit(Orbit*);
-    astro::ReferencePlane getOrbitReferencePlane() const;
-    void setOrbitReferencePlane(astro::ReferencePlane);
-    RotationElements getRotationElements() const;
-    void setRotationElements(const RotationElements&);
+    const Body* getOrbitBarycenter() const;
+    void setOrbitBarycenter(const Body*);
+
+    const ReferenceFrame* getOrbitFrame() const;
+    void setOrbitFrame(const ReferenceFrame* f);
+    const ReferenceFrame* getBodyFrame() const;
+    void setBodyFrame(const ReferenceFrame* f);
+
+    const RotationModel* getRotationModel() const;
+    void setRotationModel(const RotationModel*);
     float getRadius() const;
     void setRadius(float);
     float getMass() const;
@@ -139,26 +155,53 @@ class Body
                         float distanceFromSun) const;
     float getLuminosity(float sunLuminosity,
                         float distanceFromSun) const;
+
+    /*! Get the apparent magnitude of the body, neglecting the phase (as if
+     *  the body was at opposition.
+     */
     float getApparentMagnitude(const Star& sun,
                                float distanceFromSun,
                                float distanceFromViewer) const;
+
+    /*! Get the apparent magnitude of the body, neglecting the phase (as if
+     *  the body was at opposition.
+     */
+    float getApparentMagnitude(float sunLuminosity,
+                               float distanceFromSun,
+                               float distanceFromViewer) const;
+
+    /*! Get the apparent magnitude of the body, corrected for its phase.
+     */
     float getApparentMagnitude(const Star& sun,
                                const Vec3d& sunPosition,
                                const Vec3d& viewerPosition) const;
+
+    /*! Get the apparent magnitude of the body, corrected for the phase.
+     */
     float getApparentMagnitude(float sunLuminosity,
                                const Vec3d& sunPosition,
                                const Vec3d& viewerPosition) const;
 
+    /*! Get the transformation which converts body coordinates into
+     *  heliocentric coordinates. Some clarification on the meaning
+     *  of 'heliocentric': the position of every solar system body
+     *  is ultimately defined with respect to some star or star
+     *  system barycenter. Currently, this star is the root of the
+     *  name hierarchy containing the body. In future (post-1.5.0)
+     *  versions of Celestia, this will be changed so that the
+     *  reference star is the root of the frame hierarchy.
+     */
     Mat4d getLocalToHeliocentric(double) const;
     Point3d getHeliocentricPosition(double) const;
-    Quatd getEquatorialToGeographic(double) const;
+    Quatd getEquatorialToBodyFixed(double) const;
+    Quatd getEclipticalToFrame(double) const;
     Quatd getEclipticalToEquatorial(double) const;
-    Quatd getEclipticalToGeographic(double) const;
-    Mat4d getGeographicToHeliocentric(double) const;
+    Quatd getEclipticalToBodyFixed(double) const;
+    Mat4d getBodyFixedToHeliocentric(double) const;
 
-    Vec3f planetocentricToCartesian(float lon, float lat, float alt) const;
-    Vec3f planetocentricToCartesian(const Vec3f& lonLatAlt) const;
-    Vec3f cartesianToPlanetocentric(const Vec3f& v) const;
+    Vec3d planetocentricToCartesian(double lon, double lat, double alt) const;
+    Vec3d planetocentricToCartesian(const Vec3d& lonLatAlt) const;
+    Vec3d cartesianToPlanetocentric(const Vec3d& v) const;
 
     bool extant(double) const;
     void setLifespan(double, double);
@@ -173,14 +216,34 @@ class Body
     Location* findLocation(const std::string&, bool i18n = false) const;
     void computeLocations();
 
+    enum
+    {
+        BodyAxes       =   0x01,
+        FrameAxes      =   0x02,
+        LongLatGrid    =   0x04,
+        SunDirection   =   0x08,
+        VelocityVector =   0x10,
+    };
+	
+    bool referenceMarkVisible(uint32) const;
+    uint32 getVisibleReferenceMarks() const;
+    void setVisibleReferenceMarks(uint32);
+
+    Star* getReferenceStar() const;
+    Star* getFrameReferenceStar() const;
+
  private:
     std::string name;
     std::string i18nName;
 
     PlanetarySystem* system;
+    
     Orbit* orbit;
-    astro::ReferencePlane orbitRefPlane;
-    RotationElements rotationElements;
+    const Body* orbitBarycenter;
+    const ReferenceFrame* orbitFrame;
+    const ReferenceFrame* bodyFrame;
+    
+    const RotationModel* rotationModel;
 
     float radius;
     float mass;
@@ -208,6 +271,11 @@ class Body
 
     std::vector<Location*>* locations;
     mutable bool locationsComputed;
+
+    uint32 referenceMarks;
+
+    // Only necessary until we switch to using frame hierarchy
+    Star* frameRefStar;
 };
 
 #endif // _CELENGINE_BODY_H_
