@@ -35,7 +35,7 @@ class InstantaneousCommand : public Command
     virtual ~InstantaneousCommand() {};
     virtual double getDuration() const { return 0.0; };
     virtual void process(ExecutionEnvironment&) = 0;
-    void process(ExecutionEnvironment& env, double t, double dt)
+    void process(ExecutionEnvironment& env, double /*t*/, double /*dt*/)
     {
         process(env);
     };
@@ -94,10 +94,10 @@ class CommandGoto : public InstantaneousCommand
 class CommandGotoLongLat : public InstantaneousCommand
 {
  public:
-    CommandGotoLongLat::CommandGotoLongLat(double t,
-                                           double dist,
-                                           float _longitude, float _latitude,
-                                           Vec3f _up);
+    CommandGotoLongLat(double t,
+                       double dist,
+                       float _longitude, float _latitude,
+                       Vec3f _up);
     ~CommandGotoLongLat();
     void process(ExecutionEnvironment&);
 
@@ -469,7 +469,7 @@ class CommandPreloadTextures : public InstantaneousCommand
 class CommandMark : public InstantaneousCommand
 {
  public:
-    CommandMark(const std::string&, Color, float, Marker::Symbol);
+    CommandMark(const std::string&, Color, float, Marker::Symbol, const std::string&);
     void process(ExecutionEnvironment&);
 
  private:
@@ -477,6 +477,7 @@ class CommandMark : public InstantaneousCommand
     Color color;
     float size;
     Marker::Symbol symbol;
+    std::string label;
 };
 
 
