@@ -244,7 +244,7 @@ void EpochConvert(double jdFrom, double jdTo,
 {
     double T = (jdFrom - astro::J2000) / 36525.0;
     double t = (jdTo - jdFrom) / 36525.0;
-            
+
     double zeta = (2306.2181 + 1.39656 * T - 0.000139 * T * T) * t +
         (0.30188 - 0.000344 * T) * t * t + 0.017998 * t * t * t;
     double z = (2306.2181 + 1.39656 * T - 0.000139 * T * T) * t +
@@ -273,7 +273,7 @@ double meanAnomalySun(double t)
 	t2 = t*t;
 	a = 9.999736042e1*t;
 	b = 360*(a - (int)a);
-	
+
     return degToRad(3.5847583e2 - (1.5e-4 + 3.3e-6*t)*t2 + b);
 }
 
@@ -381,6 +381,9 @@ void ComputeGalileanElements(double t,
 
 class MercuryOrbit : public CachingOrbit
 {
+ public:
+    virtual ~MercuryOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     const int p = 0;  //Planet 0
@@ -443,6 +446,9 @@ class MercuryOrbit : public CachingOrbit
 
 class VenusOrbit : public CachingOrbit
 {
+ public:
+    virtual ~VenusOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     const int p = 1;  //Planet 1
@@ -513,6 +519,9 @@ class VenusOrbit : public CachingOrbit
 
 class EarthOrbit : public CachingOrbit
 {
+ public:
+    virtual ~EarthOrbit() {};
+
     Point3d computePosition(double jd) const
     {
    	double t, t2;
@@ -558,7 +567,7 @@ class EarthOrbit : public CachingOrbit
 
         // Correction for internal coordinate system
         eclLong += PI;
-        
+
         return Point3d(-cos(eclLong) * distance,
                        0,
                        sin(eclLong) * distance);
@@ -575,8 +584,12 @@ class EarthOrbit : public CachingOrbit
     };
 };
 
+
 class LunarOrbit : public CachingOrbit
 {
+ public:
+    virtual ~LunarOrbit() {};
+
     Point3d computePosition(double jd) const
     {
 	double jd19, t, t2;
@@ -704,11 +717,12 @@ class LunarOrbit : public CachingOrbit
         // Now compute distance using horizontal parallax.
         distance = 6378.14 / sin(horzPar);
 
+#if 1
         // Finally convert eclLat, eclLon to RA, Dec.
         EclipticToEquatorial(t, eclLat, eclLon, RA, dec);
 
         // RA and Dec are referred to the equinox of date; we want to use
-        // the J2000 equinox instead.  A better idea would be to directly 
+        // the J2000 equinox instead.  A better idea would be to directly
         // compute the position of the Moon in this coordinate system, but
         // this was easier.
         EpochConvert(jd, astro::J2000, RA, dec, RA, dec);
@@ -720,6 +734,14 @@ class LunarOrbit : public CachingOrbit
         return Point3d(cos(RA) * sin(dec) * distance,
                        cos(dec) * distance,
                        -sin(RA) * sin(dec) * distance);
+#else
+        // Skip the conversion and return ecliptical coordinates
+        double x = distance * cos(eclLat) * cos(eclLon);
+        double y = distance * cos(eclLat) * sin(eclLon);
+        double z = distance * sin(eclLat);
+
+        return Point3d(x, z, -y);
+#endif
     };
 
     double getPeriod() const
@@ -733,8 +755,12 @@ class LunarOrbit : public CachingOrbit
     };
 };
 
+
 class MarsOrbit : public CachingOrbit
 {
+ public:
+    virtual ~MarsOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     const int p = 2;  //Planet 2
@@ -816,6 +842,9 @@ class MarsOrbit : public CachingOrbit
 
 class JupiterOrbit : public CachingOrbit
 {
+ public:
+    virtual ~JupiterOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     const int p = 3;  //Planet 3
@@ -924,6 +953,9 @@ class JupiterOrbit : public CachingOrbit
 
 class SaturnOrbit : public CachingOrbit
 {
+ public:
+    virtual ~SaturnOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     const int p = 4;  //Planet 4
@@ -1059,6 +1091,9 @@ class SaturnOrbit : public CachingOrbit
 
 class UranusOrbit : public CachingOrbit
 {
+ public:
+    virtual ~UranusOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     const int p = 5;  //Planet 5
@@ -1149,6 +1184,9 @@ class UranusOrbit : public CachingOrbit
 
 class NeptuneOrbit : public CachingOrbit
 {
+ public:
+    virtual ~NeptuneOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     const int p = 6;  //Planet 6
@@ -1229,6 +1267,9 @@ class NeptuneOrbit : public CachingOrbit
 
 class PlutoOrbit : public CachingOrbit
 {
+ public:
+    virtual ~PlutoOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     const int p = 7;  //Planet 7
@@ -1290,6 +1331,9 @@ static Point3d ellipsePosition(double a, double e, double M)
 
 class PhobosOrbit : public CachingOrbit
 {
+ public:
+    virtual ~PhobosOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         double epoch = 2433283.0 - 0.5; // 00:00 1 Jan 1950
@@ -1341,7 +1385,7 @@ class PhobosOrbit : public CachingOrbit
 
         return RMars_eq * (RLaplacian * (Rorbit * p));
     }
-    
+
     double getPeriod() const
     {
         return 0.319;
@@ -1356,6 +1400,9 @@ class PhobosOrbit : public CachingOrbit
 
 class DeimosOrbit : public CachingOrbit
 {
+ public:
+    virtual ~DeimosOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         double epoch = 2433283.0 - 0.5;
@@ -1436,7 +1483,7 @@ class DeimosOrbit : public CachingOrbit
 
         // Compute the mean anomaly
         double M = L - P;
-        
+
         // Solve Kepler's equation--for a low eccentricity orbit, just a few
         // iterations is enough.
         double E = M;
@@ -1495,6 +1542,9 @@ static const double JupAscendingNode = degToRad(22.203);
 
 class IoOrbit : public CachingOrbit
 {
+ public:
+    virtual ~IoOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     //Computation will yield latitude(L), longitude(B) and distance(R) relative to Jupiter
@@ -1573,6 +1623,9 @@ class IoOrbit : public CachingOrbit
 
 class EuropaOrbit : public CachingOrbit
 {
+ public:
+    virtual ~EuropaOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     // Computation will yield latitude(L), longitude(B) and distance(R) relative to Jupiter
@@ -1662,6 +1715,9 @@ class EuropaOrbit : public CachingOrbit
 
 class GanymedeOrbit : public CachingOrbit
 {
+ public:
+    virtual ~GanymedeOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     //Computation will yield latitude(L), longitude(B) and distance(R) relative to Jupiter
@@ -1754,6 +1810,9 @@ class GanymedeOrbit : public CachingOrbit
 
 class CallistoOrbit : public CachingOrbit
 {
+ public:
+    virtual ~CallistoOrbit() {};
+
     Point3d computePosition(double jd) const
     {
     //Computation will yield latitude(L), longitude(B) and distance(R) relative to Jupiter
@@ -1860,7 +1919,7 @@ class CallistoOrbit : public CachingOrbit
         - 8.9e-6    * cos(l4 - LPEJ -G)
         - 6.2e-6    * cos(l4 + p4 - 2*LPEJ - 3*G)
         + 4.8e-6    * cos(2*(l4 - w4));
-    
+
     R = 26.36273 * JupiterRadius * (1 + R);
 
     T = (jd - 2433282.423) / 36525.0;
@@ -1986,6 +2045,9 @@ static void OuterSaturnMoonParams(double a, double e, double i,
 
 class MimasOrbit : public CachingOrbit
 {
+ public:
+    virtual ~MimasOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Computation will yield latitude(L), longitude(B) and distance(R)
@@ -2025,6 +2087,9 @@ class MimasOrbit : public CachingOrbit
 
 class EnceladusOrbit : public CachingOrbit
 {
+ public:
+    virtual ~EnceladusOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Computation will yield latitude(L), longitude(B) and distance(R)
@@ -2063,6 +2128,9 @@ class EnceladusOrbit : public CachingOrbit
 
 class TethysOrbit : public CachingOrbit
 {
+ public:
+    virtual ~TethysOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Computation will yield latitude(L), longitude(B) and distance(R)
@@ -2097,6 +2165,9 @@ class TethysOrbit : public CachingOrbit
 
 class DioneOrbit : public CachingOrbit
 {
+ public:
+    virtual ~DioneOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Computation will yield latitude(L), longitude(B) and distance(R)
@@ -2136,6 +2207,9 @@ class DioneOrbit : public CachingOrbit
 
 class RheaOrbit : public CachingOrbit
 {
+ public:
+    virtual ~RheaOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Computation will yield latitude(L), longitude(B) and distance(R)
@@ -2146,7 +2220,7 @@ class RheaOrbit : public CachingOrbit
         ComputeSaturnianElements(jd,
                                  t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
                                  W0, W1, W2, W3, W4, W5, W6, W7, W8);
-        double e1 = 0.05589 - 0.000346 * t7;
+        /*double e1 = 0.05589 - 0.000346 * t7;  Unused*/
 
         double p_ = 342.7 + 10.057 * t2;
         double a1 = 0.000265 * sinD(p_) + 0.01 * sinD(W4);
@@ -2182,6 +2256,9 @@ class RheaOrbit : public CachingOrbit
 
 class TitanOrbit : public CachingOrbit
 {
+ public:
+    virtual ~TitanOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Computation will yield latitude(L), longitude(B) and distance(R)
@@ -2247,6 +2324,9 @@ class TitanOrbit : public CachingOrbit
 
 class HyperionOrbit : public CachingOrbit
 {
+ public:
+    virtual ~HyperionOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Computation will yield latitude(L), longitude(B) and distance(R)
@@ -2286,7 +2366,7 @@ class HyperionOrbit : public CachingOrbit
             0.3573 * sinD(zeta - eta) -
             12.872 * sinD(zeta) +
             1.668 * sinD(2 * zeta) -
-            0.2419 * sinD(3 * zeta) - 
+            0.2419 * sinD(3 * zeta) -
             0.07 * sinD(phi);
         double lam_ = 177.047 +
             16.91993829 * t6 +
@@ -2327,6 +2407,9 @@ class HyperionOrbit : public CachingOrbit
 
 class IapetusOrbit : public CachingOrbit
 {
+ public:
+    virtual ~IapetusOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Computation will yield latitude(L), longitude(B) and distance(R)
@@ -2378,7 +2461,7 @@ class IapetusOrbit : public CachingOrbit
             0.01632 * sinD(l + u2) +
             0.03547 * sinD(u4);
         double p = om0 + W / e_;
-        double lam_ = mu - 
+        double lam_ = mu -
             0.04299 * sinD(u2) -
             0.00789 * sinD(u1) -
             0.06312 * sinD(ls) -
@@ -2416,6 +2499,9 @@ class IapetusOrbit : public CachingOrbit
 
 class PhoebeOrbit : public CachingOrbit
 {
+ public:
+    virtual ~PhoebeOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         double t = jd - 2433282.5;
@@ -2449,7 +2535,7 @@ class PhoebeOrbit : public CachingOrbit
 
 class UranianSatelliteOrbit : public CachingOrbit
 {
-private:
+ private:
     double a;
     double n;
     double L0;
@@ -2461,7 +2547,7 @@ private:
     double *zeta_k, *zeta_theta, *zeta_phi;
     int zetaTerms;
 
-public:
+ public:
     UranianSatelliteOrbit(double _a,
                           double _n,
                           double _L0, double _L1,
@@ -2471,14 +2557,17 @@ public:
                           double* _zeta_k, double* _zeta_theta,
                           double* _zeta_phi) :
         a(_a), n(_n), L0(_L0), L1(_L1),
-        LTerms(_LTerms), zTerms(_zTerms), zetaTerms(_zetaTerms),
         L_k(_L_k), L_theta(_L_theta), L_phi(_L_phi),
+        LTerms(_LTerms),
         z_k(_z_k), z_theta(_z_theta), z_phi(_z_phi),
-        zeta_k(_zeta_k), zeta_theta(_zeta_theta), zeta_phi(_zeta_phi)
+        zTerms(_zTerms),
+        zeta_k(_zeta_k), zeta_theta(_zeta_theta), zeta_phi(_zeta_phi),
+        zetaTerms(_zetaTerms)
     {
     };
 
-public:
+    virtual ~UranianSatelliteOrbit() {};
+
     double getPeriod() const
     {
         return 2 * PI / n;
@@ -2572,7 +2661,7 @@ static double uran_L_phi[5][3] = {
 { 1.32, 2.64, 0 },
 { 1.32, 0, 0 },
 { 0, 0, 0 },
-{ 0, 0, 0 },   
+{ 0, 0, 0 },
 };
 static double uran_z_k[5][5] = {
 { 1.31238e-3, -1.2331e-4, -1.9410e-4, 0, 0 },
@@ -2635,6 +2724,9 @@ static UranianSatelliteOrbit* CreateUranianSatelliteOrbit(int n)
 
 class TritonOrbit : public CachingOrbit
 {
+ public:
+    virtual ~TritonOrbit() {};
+
     Point3d computePosition(double jd) const
     {
         // Pole of the fixed reference plane
@@ -2651,7 +2743,7 @@ class TritonOrbit : public CachingOrbit
         double nepPole_Dec = degToRad(43.46);
 
         double a = 354800;
-        double e = 0.0000;
+        /*double e = 0.0000;    Unused*/
         double n = degToRad(61.2588532);
         double L0 = degToRad(200.913);
         double L  = L0 + n * t;
@@ -2681,7 +2773,7 @@ class TritonOrbit : public CachingOrbit
 
         return RNept_eq * (Rrefplane * (Rorbit * p));
     }
-    
+
     double getPeriod() const
     {
         return 5.877;
@@ -2696,16 +2788,21 @@ class TritonOrbit : public CachingOrbit
 
 class JPLEphOrbit : public CachingOrbit
 {
-public:
-    JPLEphOrbit(const JPLEphemeris& e, JPLEphemItem item,
+ public:
+    JPLEphOrbit(const JPLEphemeris& e,
+                JPLEphemItem _target,
+                JPLEphemItem _center,
                 double _period, double _boundingRadius) :
         ephem(e),
-        body(item),
+        target(_target),
+        center(_center),
         period(_period),
         boundingRadius(_boundingRadius)
     {
     };
 
+    virtual ~JPLEphOrbit() {};
+
     double getPeriod() const
     {
         return period;
@@ -2716,38 +2813,63 @@ public:
         return boundingRadius;
     }
 
-    Point3d computePosition(double jd) const
+    Point3d computePosition(double tjd) const
     {
-        Point3d baryPos = ephem.getPlanetPosition(body, jd);
+        // Get the position relative to the Earth (for the Moon) or
+        // the solar system barycenter.
+        Point3d pos = ephem.getPlanetPosition(target, tjd);
 
-        // Convert from solar system barycentric position to heliocentric
-        // position.
-        Point3d sunPos = ephem.getPlanetPosition(JPLEph_Sun, jd);
-        Point3d heliocentricPos = Point3d(0.0, 0.0, 0.0) + (baryPos - sunPos);
+        if (center == JPLEph_SSB && target != JPLEph_Moon)
+        {
+            // No translation necessary
+        }
+        else if (center == JPLEph_Earth && target == JPLEph_Moon)
+        {
+            // No translation necessary
+        }
+        else
+        {
+            Point3d centerPos = ephem.getPlanetPosition(center, tjd);
+            if (target == JPLEph_Moon)
+            {
+                pos += (ephem.getPlanetPosition(JPLEph_Earth, tjd) - Point3d(0.0, 0.0, 0.0));
+            }
+            if (center == JPLEph_Moon)
+            {
+                centerPos += (ephem.getPlanetPosition(JPLEph_Earth, tjd) - Point3d(0.0, 0.0, 0.0));
+            }
+
+            // Compute the position of target relative to the center
+            pos = Point3d(pos.x - centerPos.x,
+                          pos.y - centerPos.y,
+                          pos.z - centerPos.z);
+        }
 
         // Rotate from the J2000 mean equator to the ecliptic
-        // 23 deg 26' 21".448
-        double J2000_equator = degToRad(23.4392911);
-        heliocentricPos = heliocentricPos * Mat3d::xrotation(J2000_equator);
+        pos = pos * Mat3d::xrotation(astro::J2000Obliquity);
 
-        return Point3d(heliocentricPos.x,
-                       heliocentricPos.z,
-                       -heliocentricPos.y);
+        // Convert to Celestia's coordinate system
+        return Point3d(pos.x, pos.z, -pos.y);
     }
 
-private:
+ private:
     const JPLEphemeris& ephem;
-    JPLEphemItem body;
+    JPLEphemItem target;
+    JPLEphemItem center;
     double period;
     double boundingRadius;
 };
 
 
-static Orbit* CreateJPLEphOrbit(JPLEphemItem body,
+static Orbit* CreateJPLEphOrbit(JPLEphemItem target,
+                                JPLEphemItem center,
                                 double period,
                                 double boundingRadius)
 {
-    Orbit* o = new JPLEphOrbit(*jpleph, body, period, boundingRadius);
+    if (jpleph == NULL)
+        return NULL;
+
+    Orbit* o = new JPLEphOrbit(*jpleph, target, center, period, boundingRadius);
     return new MixedOrbit(o,
                           jpleph->getStartDate(),
                           jpleph->getEndDate(),
@@ -2800,26 +2922,64 @@ Orbit* GetCustomOrbit(const string& name)
         return new MixedOrbit(new NeptuneOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "pluto")
         return new MixedOrbit(new PlutoOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
-    if (name == "mercury-jpl")
-        return CreateJPLEphOrbit(JPLEph_Mercury,  0.2408  * 365.25, 6.0e7);
-    if (name == "venus-jpl")
-        return CreateJPLEphOrbit(JPLEph_Venus,    0.6152  * 365.25, 1.0e8);
-    if (name == "earth-jpl")
-        return CreateJPLEphOrbit(JPLEph_EarthMoonBary,      365.25, 1.6e8);
-    if (name == "mars-jpl")
-        return CreateJPLEphOrbit(JPLEph_Mars,      1.8809 * 365.25, 2.4e8);
-    if (name == "jupiter-jpl")
-        return CreateJPLEphOrbit(JPLEph_Jupiter,  11.86   * 365.25, 8.0e8);
-    if (name == "saturn-jpl")
-        return CreateJPLEphOrbit(JPLEph_Saturn,   29.4577 * 365.25, 1.5e9);
-    if (name == "uranus-jpl")
-        return CreateJPLEphOrbit(JPLEph_Uranus,   84.0139 * 365.25, 3.0e9);
-    if (name == "neptune-jpl")
-        return CreateJPLEphOrbit(JPLEph_Neptune, 164.793  * 365.25, 4.7e9);
-    if (name == "pluto-jpl")
-        return CreateJPLEphOrbit(JPLEph_Pluto,   248.54   * 365.25, 6.0e9);
-    if (name == "moon-jpl")
-        return CreateJPLEphOrbit(JPLEph_Moon,     27.321661,        5.0e5);
+
+    // Two styles of custom orbit name are permitted for JPL ephemeris orbits.
+    // The preferred is <ephemeris>-<object>, e.g. jpl-mercury. But the reverse
+    // form is still supported for backward compatibility.
+    if (name == "jpl-mercury-sun" || name == "mercury-jpl")
+        return CreateJPLEphOrbit(JPLEph_Mercury, JPLEph_Sun,  0.2408  * 365.25, 6.0e7);
+    if (name == "jpl-venus-sun" || name == "venus-jpl")
+        return CreateJPLEphOrbit(JPLEph_Venus,   JPLEph_Sun,  0.6152  * 365.25, 1.0e8);
+    if (name == "jpl-earth-sun" || name == "earth-jpl")
+        return CreateJPLEphOrbit(JPLEph_Earth,   JPLEph_Sun,            365.25, 1.6e8);
+    if (name == "jpl-mars-sun" || name == "mars-jpl")
+        return CreateJPLEphOrbit(JPLEph_Mars,    JPLEph_Sun,  1.8809 * 365.25, 2.4e8);
+    if (name == "jpl-jupiter-sun" || name == "jupiter-jpl")
+        return CreateJPLEphOrbit(JPLEph_Jupiter, JPLEph_Sun, 11.86   * 365.25, 8.0e8);
+    if (name == "jpl-saturn-sun" || name == "saturn-jpl")
+        return CreateJPLEphOrbit(JPLEph_Saturn,  JPLEph_Sun, 29.4577 * 365.25, 1.5e9);
+    if (name == "jpl-uranus-sun" || name == "uranus-jpl")
+        return CreateJPLEphOrbit(JPLEph_Uranus,  JPLEph_Sun, 84.0139 * 365.25, 3.0e9);
+    if (name == "jpl-neptune-sun" || name == "neptune-jpl")
+        return CreateJPLEphOrbit(JPLEph_Neptune, JPLEph_Sun, 164.793  * 365.25, 4.7e9);
+    if (name == "jpl-pluto-sun" || name == "pluto-jpl")
+        return CreateJPLEphOrbit(JPLEph_Pluto,   JPLEph_Sun, 248.54   * 365.25, 6.0e9);
+
+    if (name == "jpl-mercury-ssb")
+        return CreateJPLEphOrbit(JPLEph_Mercury, JPLEph_SSB,  0.2408  * 365.25, 6.0e7);
+    if (name == "jpl-venus-ssb")
+        return CreateJPLEphOrbit(JPLEph_Venus,   JPLEph_SSB,  0.6152  * 365.25, 1.0e8);
+    if (name == "jpl-earth-ssb")
+        return CreateJPLEphOrbit(JPLEph_Earth,   JPLEph_SSB,            365.25, 1.6e8);
+    if (name == "jpl-mars-ssb")
+        return CreateJPLEphOrbit(JPLEph_Mars,    JPLEph_SSB,  1.8809 * 365.25, 2.4e8);
+    if (name == "jpl-jupiter-ssb")
+        return CreateJPLEphOrbit(JPLEph_Jupiter, JPLEph_SSB, 11.86   * 365.25, 8.0e8);
+    if (name == "jpl-saturn-ssb")
+        return CreateJPLEphOrbit(JPLEph_Saturn,  JPLEph_SSB, 29.4577 * 365.25, 1.5e9);
+    if (name == "jpl-uranus-ssb")
+        return CreateJPLEphOrbit(JPLEph_Uranus,  JPLEph_SSB, 84.0139 * 365.25, 3.0e9);
+    if (name == "jpl-neptune-ssb")
+        return CreateJPLEphOrbit(JPLEph_Neptune, JPLEph_SSB, 164.793  * 365.25, 4.7e9);
+    if (name == "jpl-pluto-ssb")
+        return CreateJPLEphOrbit(JPLEph_Pluto,   JPLEph_SSB, 248.54   * 365.25, 6.0e9);
+
+
+    // JPL ephemerides for Earth-Moon system
+    if (name == "jpl-emb-sun") // Earth-Moon barycenter, heliocentric
+        return CreateJPLEphOrbit(JPLEph_EarthMoonBary, JPLEph_Sun,      365.25, 1.6e8);
+    if (name == "jpl-emb-ssb") // Earth-Moon barycenter, relative to ssb
+        return CreateJPLEphOrbit(JPLEph_EarthMoonBary, JPLEph_SSB,      365.25, 1.6e8);
+    if (name == "jpl-moon-emb") // Moon, barycentric
+        return CreateJPLEphOrbit(JPLEph_Moon, JPLEph_EarthMoonBary,      27.321661,        5.0e5);
+    if (name == "jpl-moon-earth") // Moon, geocentric
+        return CreateJPLEphOrbit(JPLEph_Moon,    JPLEph_Earth,           27.321661,        5.0e5);
+    if (name == "jpl-earth-emb") // Earth, barycentric
+        return CreateJPLEphOrbit(JPLEph_Earth,   JPLEph_EarthMoonBary,   27.321,           1.0e5);
+                                 
+    if (name == "jpl-sun-ssb")   // Position of Sun relative to SSB
+        return CreateJPLEphOrbit(JPLEph_Sun,     JPLEph_SSB,             11.861773 * 365.25, 2000000);
+
     if (name == "phobos")
         return new PhobosOrbit();
     if (name == "deimos")
