@@ -18,6 +18,7 @@
 #include "opencluster.h"
 #include <celutil/util.h>
 #include <celutil/debug.h>
+#include <celmath/intersect.h>
 
 using namespace std;
 
@@ -107,6 +108,15 @@ void DeepSkyObject::setInfoURL(const string& s)
         *infoURL = s;
 }
 
+
+bool DeepSkyObject::pick(const Ray3d& ray,
+                         double& distanceToPicker,
+                         double& cosAngleToBoundCenter) const
+{
+    return testIntersection(ray, Sphered(position, (double) radius), distanceToPicker, cosAngleToBoundCenter);
+}
+
+
 bool DeepSkyObject::load(AssociativeArray* params, const string& resPath)
 {
     // Get position
@@ -136,7 +146,7 @@ bool DeepSkyObject::load(AssociativeArray* params, const string& resPath)
     q.setAxisAngle(Vec3f((float) axis.x, (float) axis.y, (float) axis.z),
                    (float) degToRad(angle));
     setOrientation(q);
-    
+
     double radius = 1.0;
     params->getNumber("Radius", radius);
     setRadius((float) radius);
@@ -147,7 +157,18 @@ bool DeepSkyObject::load(AssociativeArray* params, const string& resPath)
 
     string infoURL;
     if (params->getString("InfoURL", infoURL))
+    {
+        if (infoURL.find(':') == string::npos)
+        {
+            // Relative URL, the base directory is the current one,
+            // not the main installation directory
+            if (resPath[1] == ':')
+                // Absolute Windows path, file:/// is required
+                infoURL = "file:///" + resPath + "/" + infoURL;
+            else if (!resPath.empty())
+                infoURL = resPath + "/" + infoURL;
+        }
         setInfoURL(infoURL);
-
+    }
     return true;
 }
