@@ -12,10 +12,10 @@
 
 #ifndef _WIN32
 // Assume that this is a UNIX/X11 system if it's not Windows or Mac OS X.
-#ifndef MACOSX
+#ifndef TARGET_OS_MAC
 #include "GL/glx.h"
 #include <dlfcn.h>
-#endif /* ! MACOSX */
+#endif /* ! TARGET_OS_MAC */
 #endif /* ! _WIN32 */
 
 #include "glext.h"
@@ -130,6 +130,11 @@ glx::PFNGLBLENDEQUATIONEXTPROC glx::glBlendEquationEXT;
 glx::PFNWGLSWAPINTERVALEXTPROC glx::wglSwapIntervalEXT;
 glx::PFNWGLGETSWAPINTERVALEXTPROC glx::wglGetSwapIntervalEXT;
 
+// GLX_SGI_video_sync command function pointers
+glx::PFNGLXGETVIDEOSYNCSGIPROC glx::glXGetVideoSyncSGI;
+glx::PFNGLXWAITVIDEOSYNCSGIPROC glx::glXWaitVideoSyncSGI;
+glx::PFNGLXGETREFRESHRATESGIPROC glx::glXGetRefreshRateSGI;
+
 // ARB_vertex_program function pointers
 glx::PFNGLBINDPROGRAMARBPROC glx::glBindProgramARB;
 glx::PFNGLDELETEPROGRAMSARBPROC glx::glDeleteProgramsARB;
@@ -270,7 +275,16 @@ glx::PFNGLGETSHADERSOURCEARBPROC           glx::glGetShaderSourceARB;
 
 // extern void Alert(const char *szFormat, ...);
 
-#ifndef MACOSX
+#if defined(QT_OPENGL_LIB) && defined(_WIN32) && !defined(_MSC_VER)
+
+// TODO: We should use something other than a global GL context
+#include <QGLContext>
+extern QGLContext* glctx;
+#define GET_GL_PROC_ADDRESS(name) glctx->getProcAddress(name)
+
+#else
+
+#ifndef TARGET_OS_MAC
 #if defined(_WIN32)
 
 #define GET_GL_PROC_ADDRESS(name) wglGetProcAddress(name)
@@ -304,8 +318,12 @@ FUNCS GetGLProcAddress(const GLubyte* procName)
 }
 
 #endif // defined(WIN32)
-#endif /* !MACOSX */
-#ifdef MACOSX
+#endif // !TARGET_OS_MAC
+
+#endif // QT_CORE_LIB
+
+
+#ifdef TARGET_OS_MAC
 #include <mach-o/dyld.h>
 #include <stdio.h>
 typedef void (*FUNCS) (void);
@@ -334,7 +352,7 @@ FUNCS osxGetProcAddress(const GLubyte *procName) {
         return NULL;
     }
     strcpy(myProcName,"_");
-    
+
     /* sanity check */
     if (strlen((char *)procName)>125) return NULL;
     strcat(myProcName,(char *)procName);
@@ -348,10 +366,10 @@ FUNCS osxGetProcAddress(const GLubyte *procName) {
     //}
 }
 #define GET_GL_PROC_ADDRESS(name) osxGetProcAddress((GLubyte *)name)
-#endif /* MACOSX */
+#endif /* TARGET_OS_MAC */
 
 
-void Alert(const char *szFormat, ...)
+void Alert(const char /* *szFormat*/, ...)
 {
 }
 
@@ -650,6 +668,14 @@ static void InitExt_EXT_swap_control()
 }
 
 
+static void InitExt_GLX_SGI_video_sync()
+{
+    glx::glXGetVideoSyncSGI = (glx::PFNGLXGETVIDEOSYNCSGIPROC) GET_GL_PROC_ADDRESS("glXGetVideoSyncSGI");
+    glx::glXWaitVideoSyncSGI = (glx::PFNGLXWAITVIDEOSYNCSGIPROC) GET_GL_PROC_ADDRESS("glXWaitVideoSyncSGI");
+    glx::glXGetRefreshRateSGI = (glx::PFNGLXGETREFRESHRATESGIPROC) GET_GL_PROC_ADDRESS("glXGetRefreshRateSGI");
+}
+
+
 static void InitExt_NV_fragment_program()
 {
 #if defined(GET_GL_PROC_ADDRESS)
@@ -840,6 +866,9 @@ void InitExtension(const char* ext)
         InitExt_ARB_vertex_shader();
     else if (!strcmp(ext, "WGL_EXT_swap_control"))
         InitExt_EXT_swap_control();
+    else if (!strcmp(ext, "GLX_SGI_video_sync"))
+        InitExt_GLX_SGI_video_sync();
+
 }
 
 
