@@ -141,20 +141,20 @@ namespace glx
     extern PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB;
 };
 
-/* EXT_texture_cube_map defines from <GL/gl.h> */
-#if !(defined( GL_EXT_texture_cube_map) || defined(__glext_h_)) || defined(MACOSX)
-#define GL_NORMAL_MAP_EXT                   ((GLenum) 0x8511)
-#define GL_REFLECTION_MAP_EXT               ((GLenum) 0x8512)
-#define GL_TEXTURE_CUBE_MAP_EXT             ((GLenum) 0x8513)
-#define GL_TEXTURE_BINDING_CUBE_MAP_EXT     ((GLenum) 0x8514)
-#define GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT  ((GLenum) 0x8515)
-#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT  ((GLenum) 0x8516)
-#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT  ((GLenum) 0x8517)
-#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT  ((GLenum) 0x8518)
-#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT  ((GLenum) 0x8519)
-#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT  ((GLenum) 0x851A)
-#define GL_PROXY_TEXTURE_CUBE_MAP_EXT       ((GLenum) 0x851B)
-#define GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT    ((GLenum) 0x851C)
+/* ARB_texture_cube_map defines from <GL/gl.h> */
+#if !(defined( GL_ARB_texture_cube_map) || defined(__glext_h_))
+#define GL_NORMAL_MAP_ARB                   ((GLenum) 0x8511)
+#define GL_REFLECTION_MAP_ARB               ((GLenum) 0x8512)
+#define GL_TEXTURE_CUBE_MAP_ARB             ((GLenum) 0x8513)
+#define GL_TEXTURE_BINDING_CUBE_MAP_ARB     ((GLenum) 0x8514)
+#define GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB  ((GLenum) 0x8515)
+#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB  ((GLenum) 0x8516)
+#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB  ((GLenum) 0x8517)
+#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB  ((GLenum) 0x8518)
+#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB  ((GLenum) 0x8519)
+#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB  ((GLenum) 0x851A)
+#define GL_PROXY_TEXTURE_CUBE_MAP_ARB       ((GLenum) 0x851B)
+#define GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB    ((GLenum) 0x851C)
 #endif
 
 /* EXT_separate_specular_color defines from <GL/gl.h> */
@@ -405,7 +405,7 @@ namespace glx
 
 
 /* NV_vertex_program */
-#if !defined(GL_NV_vertex_program)||defined(MACOSX)
+#if !defined(GL_NV_vertex_program)||defined(TARGET_OS_MAC)
 
 #define GL_VERTEX_PROGRAM_NV              0x8620
 #define GL_VERTEX_STATE_PROGRAM_NV        0x8621
@@ -481,7 +481,7 @@ namespace glx
 #define GL_MAP2_VERTEX_ATTRIB14_4_NV      0x867E
 #define GL_MAP2_VERTEX_ATTRIB15_4_NV      0x867F
 
-#endif // !defined(GL_NV_VERTEX_PROGRAM) || defined(MACOSX)
+#endif // !defined(GL_NV_VERTEX_PROGRAM) || defined(TARGET_OS_MAC)
 
 #ifndef GL_VERTEX_ATTRIB_ARRAY6_NV
 #define GL_VERTEX_ATTRIB_ARRAY6_NV        0x8656
@@ -954,6 +954,8 @@ namespace glx
 #define GL_ARB_point_sprite 1
 #define GL_POINT_SPRITE_ARB               0x8861
 #define GL_COORD_REPLACE_ARB              0x8862
+#define GL_SMOOTH_POINT_SIZE_RANGE        0x0B12
+#define GL_ALIASED_POINT_SIZE_RANGE       0x846D
 #endif
 
 
@@ -965,6 +967,7 @@ namespace glx
 
 #ifndef GL_ARB_shading_language_100
 #define GL_ARB_shading_language_100 1
+#define GL_SHADING_LANGUAGE_VERSION_ARB   0x8B8C
 #endif
 
 namespace glx
@@ -1214,6 +1217,79 @@ namespace glx
 };
 
 
+/* GL_ARB_pixel_buffer_object */
+#ifndef GL_ARB_pixel_buffer_object
+#define GL_ARB_pixel_buffer_object        1
+#define GL_PIXEL_PACK_BUFFER_ARB          0x88EB
+#define GL_PIXEL_UNPACK_BUFFER_ARB        0x88EC
+#define GL_PIXEL_PACK_BUFFER_BINDING_ARB  0x88ED
+#define GL_PIXEL_UNPACK_BUFFER_BINDING_ARB 0x88EF
+#endif
+
+
+/* GL_ARB_color_buffer_float */
+#ifndef GL_ARB_color_buffer_float
+#define GL_ARB_color_buffer_float         1
+#define GL_RGBA_FLOAT_MODE_ARB            0x8820
+#define GL_CLAMP_VERTEX_COLOR_ARB         0x891A
+#define GL_CLAMP_FRAGMENT_COLOR_ARB       0x891B
+#define GL_CLAMP_READ_COLOR_ARB           0x891C
+#define GL_FIXED_ONLY_ARB                 0x891D
+#endif
+
+namespace glx
+{
+    typedef void (APIENTRY* PFNGLCLAMPCOLORARBPROC) (GLenum target, GLenum clamp);
+    extern PFNGLCLAMPCOLORARBPROC glClampColorARB;
+};
+
+
+/* GL_ARB_texture_float */
+#ifndef GL_ARB_texture_float
+#define GL_ARB_texture_float              1
+#define GL_TEXTURE_RED_TYPE_ARB           0x8C10
+#define GL_TEXTURE_GREEN_TYPE_ARB         0x8C11
+#define GL_TEXTURE_BLUE_TYPE_ARB          0x8C12
+#define GL_TEXTURE_ALPHA_TYPE_ARB         0x8C13
+#define GL_TEXTURE_LUMINANCE_TYPE_ARB     0x8C14
+#define GL_TEXTURE_INTENSITY_TYPE_ARB     0x8C15
+#define GL_TEXTURE_DEPTH_TYPE_ARB         0x8C16
+#define GL_UNSIGNED_NORMALIZED_ARB        0x8C17
+#define GL_RGBA32F_ARB                    0x8814
+#define GL_RGB32F_ARB                     0x8815
+#define GL_ALPHA32F_ARB                   0x8816
+#define GL_INTENSITY32F_ARB               0x8817
+#define GL_LUMINANCE32F_ARB               0x8818
+#define GL_LUMINANCE_ALPHA32F_ARB         0x8819
+#define GL_RGBA16F_ARB                    0x881A
+#define GL_RGB16F_ARB                     0x881B
+#define GL_ALPHA16F_ARB                   0x881C
+#define GL_INTENSITY16F_ARB               0x881D
+#define GL_LUMINANCE16F_ARB               0x881E
+#define GL_LUMINANCE_ALPHA16F_ARB         0x881F
+#endif
+
+
+/* WGL_ARB_pixel_format_float */
+#ifndef WGL_ARB_pixel_format_float
+#define WGL_ARB_pixel_format_float        1
+#define WGL_TYPE_RGBA_FLOAT_ARB           0x21A0
+#endif
+
+
+/* SGI Video Sync from glxext.h, for refresh-rate syncing. */
+namespace glx
+{
+    typedef int ( * PFNGLXGETVIDEOSYNCSGIPROC) (unsigned int *count);
+    typedef int ( * PFNGLXWAITVIDEOSYNCSGIPROC) (int divisor, int remainder, unsigned int *count);
+    typedef int ( * PFNGLXGETREFRESHRATESGIPROC) (unsigned int *);
+
+    extern PFNGLXGETVIDEOSYNCSGIPROC glXGetVideoSyncSGI;
+    extern PFNGLXWAITVIDEOSYNCSGIPROC glXWaitVideoSyncSGI;
+    extern PFNGLXGETREFRESHRATESGIPROC glXGetRefreshRateSGI;
+};
+
+
 extern void InitExtension(const char* ext);
 extern bool ExtensionSupported(const char *ext);
 
