@@ -36,16 +36,26 @@ class Marker
 
     enum Symbol
     {
-        Diamond  = 0,
-        Triangle = 1,
-        Square   = 2,
-        Plus     = 3,
-        X        = 4
+        Diamond    = 0,
+        Triangle     = 1,
+        Square     = 2,
+        FilledSquare = 3,
+        Plus       = 4,
+        X          = 5,
+        LeftArrow  = 6,
+        RightArrow = 7,
+        UpArrow    = 8,
+        DownArrow  = 9,
+        Circle    = 10,
+        Disk     = 11
     };
 
     Symbol getSymbol() const;
     void setSymbol(Symbol);
 
+    string getLabel() const;
+    void setLabel(string);
+
     void render() const;
 
  private:
@@ -54,6 +64,7 @@ class Marker
     Color color;
     int priority;
     Symbol symbol;
+    string label;
 };
 
 typedef std::vector<Marker> MarkerList;
