@@ -1,6 +1,6 @@
 // model.h
 //
-// Copyright (C) 2004, Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2004-2006, Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -37,6 +37,18 @@ class Model
      */
     uint32 addMaterial(const Mesh::Material*);
 
+    /*! Return the number of materials in the model
+     */
+    uint32 getMaterialCount() const;
+
+    /*! Return the total number of vertices in the model
+     */
+    uint32 getVertexCount() const;
+
+    /*! Return the total number of primitives in the model
+     */
+    uint32 getPrimitiveCount() const;
+
     /*! Return the mesh with the specified index, or NULL if the
      *  index is out of range.
      */
@@ -69,15 +81,30 @@ class Model
      */
     bool usesTextureType(Mesh::TextureSemantic) const;
 
+    /*! Return true if the model has no translucent components. */
+    bool isOpaque() const
+    {
+        return opaque;
+    }
+
+    /*! Set the opacity flag based on material usage within the model */
+    void determineOpacity();
+
+
     class MeshComparator
     {
-    public:
+     public:
+        virtual ~MeshComparator() {};
+
         virtual bool operator()(const Mesh&, const Mesh&) const = 0;
     };
 
     /*! Sort the model's meshes in place. */
     void sortMeshes(const MeshComparator&);
 
+    /*! Optimize the model by eliminating all duplicated materials */
+    void uniquifyMaterials();
+
     /*! This comparator will roughly sort the model's meshes by
      *  opacity so that transparent meshes are rendered last.  It's far
      *  from perfect, but covers a lot of cases.  A better method of
@@ -85,19 +112,22 @@ class Model
      *  even better at the triangle level.
      *
      *  Standard usage for this class is:
-     *     model->sortMeshes(Model::OpacityComparator(*model));
+     *     model->sortMeshes(Model::OpacityComparator());
+     *
+     *  uniquifyMaterials() should be used before sortMeshes(), since
+     *  the opacity comparison depends on material indices being ordered
+     *  by opacity.
      */
     class OpacityComparator : public MeshComparator
     {
-    public:
-        OpacityComparator(const Model&);
+     public:
+        OpacityComparator();
+        virtual ~OpacityComparator() {};
 
         virtual bool operator()(const Mesh&, const Mesh&) const;
 
-    private:
-        float getOpacity(const Mesh& mesh) const;
-
-        const Model& model;
+     private:
+        int unused;
     };
 
  private:
@@ -105,6 +135,7 @@ class Model
     std::vector<Mesh*> meshes;
 
     bool textureUsage[Mesh::TextureSemanticMax];
+    bool opaque;
 };
 
 #endif // !_CELENGINE_MODEL_H_
