@@ -18,6 +18,7 @@
 #include "meshmanager.h"
 #include "rendcontext.h"
 #include "gl.h"
+#include "glext.h"
 #include "vecgl.h"
 #include "render.h"
 
@@ -36,15 +37,14 @@ const char* Nebula::getType() const
 }
 
 
-void Nebula::setType(const string& typeStr)
+void Nebula::setType(const string& /*typeStr*/)
 {
 }
 
 
 size_t Nebula::getDescription(char* buf, size_t bufLength) const
 {
-    // Should use snprintf, but it's not available on Windows
-    return sprintf(buf, _("%s"), getType());
+    return snprintf(buf, bufLength, _("%s"), getType());
 }
 
 
@@ -53,11 +53,26 @@ ResourceHandle Nebula::getModel() const
     return model;
 }
 
+
 void Nebula::setModel(ResourceHandle _model)
 {
     model = _model;
 }
 
+const char* Nebula::getObjTypeName() const
+{
+    return "nebula";
+}
+
+
+bool Nebula::pick(const Ray3d& ray,
+                  double& distanceToPicker,
+                  double& cosAngleToBoundCenter) const
+{
+    // The preconditional sphere-ray intersection test is enough for now:
+    return DeepSkyObject::pick(ray, distanceToPicker, cosAngleToBoundCenter);
+}
+
 
 bool Nebula::load(AssociativeArray* params, const string& resPath)
 {
@@ -68,15 +83,15 @@ bool Nebula::load(AssociativeArray* params, const string& resPath)
             GetModelManager()->getHandle(ModelInfo(model, resPath));
         setModel(modelHandle);
     }
-    
+
     return DeepSkyObject::load(params, resPath);
 }
 
 
-void Nebula::render(const GLContext&,
-                    const Vec3f& offset,
-                    const Quatf& viewerOrientation,
-                    float brightness,
+void Nebula::render(const GLContext& glcontext,
+                    const Vec3f&,
+                    const Quatf&,
+                    float,
                     float pixelSize)
 {
     Model* m = NULL;
@@ -85,21 +100,32 @@ void Nebula::render(const GLContext&,
     if (m == NULL)
         return;
 
-    glDisable(GL_LIGHTING);
-    glEnable(GL_TEXTURE_2D);
+    glDisable(GL_BLEND);
+
     glScalef(getRadius(), getRadius(), getRadius());
     glRotate(getOrientation());
 
-    FixedFunctionRenderContext rc;
-    m->render(rc);
+    if (glcontext.getRenderPath() == GLContext::GLPath_GLSL)
+    {
+        GLSLUnlit_RenderContext rc(getRadius());
+        rc.setPointScale(2.0 * getRadius() / pixelSize);
+        m->render(rc);
+        glx::glUseProgramObjectARB(0);
+    }
+    else
+    {
+        FixedFunctionRenderContext rc;
+        rc.setLighting(false);
+        m->render(rc);
 
-    // Reset the material
-    float black[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
-    float zero = 0.0f;
-    glColor4fv(black);
-    glMaterialfv(GL_FRONT, GL_EMISSION, black);
-    glMaterialfv(GL_FRONT, GL_SPECULAR, black);
-    glMaterialfv(GL_FRONT, GL_SHININESS, &zero);
+        // Reset the material
+        float black[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
+        float zero = 0.0f;
+        glColor4fv(black);
+        glMaterialfv(GL_FRONT, GL_EMISSION, black);
+        glMaterialfv(GL_FRONT, GL_SPECULAR, black);
+        glMaterialfv(GL_FRONT, GL_SHININESS, &zero);
+    }
 
     glEnable(GL_BLEND);
 }
