@@ -25,7 +25,7 @@ class Observer
 public:
     Observer();
 
-    // The getPosition method returns the observer's position in light
+    // The getPosition method returns the observer's position in micro light
     // years.
     UniversalCoord getPosition() const;
 
@@ -56,7 +56,7 @@ public:
 
     Vec3f          getPickRay(float x, float y) const;
 
-    
+
     void orbit(const Selection&, Quatf q);
     void rotate(Quatf q);
     void changeOrbitDistance(const Selection&, float d);
@@ -66,10 +66,28 @@ public:
     Selection getTrackedObject() const;
     void setTrackedObject(const Selection&);
 
+    const std::string& getDisplayedSurface() const;
+    void setDisplayedSurface(const std::string&);
+    
+    uint32 getLocationFilter() const;
+    void setLocationFilter(uint32);
+
     void gotoSelection(const Selection&,
                        double gotoTime,
                        Vec3f up,
                        astro::CoordinateSystem upFrame);
+    void gotoSelection(const Selection&,
+                       double gotoTime,
+                       double startInter,
+                       double endInter,
+                       Vec3f up,
+                       astro::CoordinateSystem upFrame);
+    void gotoSelectionGC(const Selection&,
+                         double gotoTime,
+                         double startInter,
+                         double endInter,
+                         Vec3f up,
+                         astro::CoordinateSystem upFrame);
     void gotoSelection(const Selection&,
                        double gotoTime,
                        double distance,
@@ -86,8 +104,14 @@ public:
                              double& distance,
                              double& longitude,
                              double& latitude);
+    void gotoSelectionGC(const Selection& selection,
+                         double gotoTime,
+                         double distance,
+                         Vec3f up,
+                         astro::CoordinateSystem upFrame);
     void gotoSurface(const Selection&, double duration);
     void centerSelection(const Selection&, double centerTime = 0.5);
+    void centerSelectionCO(const Selection&, double centerTime = 0.5);
     void follow(const Selection&);
     void geosynchronousFollow(const Selection&);
     void phaseLock(const Selection&);
@@ -112,6 +136,12 @@ public:
     ObserverMode getMode() const;
     void setMode(ObserverMode);
 
+    enum TrajectoryType {
+        Linear        = 0,
+        GreatCircle   = 1,
+        CircularOrbit = 2,          
+    };
+
     struct JourneyParams
     {
         double duration;
@@ -120,22 +150,46 @@ public:
         UniversalCoord to;
         Quatf initialOrientation;
         Quatf finalOrientation;
+        double startInterpolation; // start of orientation interpolation phase [0-1]
+        double endInterpolation;   // end of orientation interpolation phase [0-1]
         double expFactor;
         double accelTime;
+        Quatf rotation1; // rotation on the CircularOrbit around centerObject
+
+        Selection centerObject;
+        
+        TrajectoryType traj;
     };
 
+    void gotoJourney(const JourneyParams&);
     // void setSimulation(Simulation* _sim) { sim = _sim; };
 
  private:
     void computeGotoParameters(const Selection& sel,
                                JourneyParams& jparams,
                                double gotoTime,
-                               Vec3d offset, astro::CoordinateSystem offsetFrame,
+                               double startInter,
+                               double endInter,
+                               Vec3d offset,
+                               astro::CoordinateSystem offsetFrame,
                                Vec3f up,
                                astro::CoordinateSystem upFrame);
+    void computeGotoParametersGC(const Selection& sel,
+                                 JourneyParams& jparams,
+                                 double gotoTime,
+                                 double startInter,
+                                 double endInter,
+                                 Vec3d offset,
+                                 astro::CoordinateSystem offsetFrame,
+                                 Vec3f up,
+                                 astro::CoordinateSystem upFrame,
+                                 const Selection& centerObj);
     void computeCenterParameters(const Selection& sel,
                                  JourneyParams& jparams,
                                  double centerTime);
+    void computeCenterCOParameters(const Selection& sel,
+                                   JourneyParams& jparams,
+                                   double centerTime);
 
  private:
     double         simTime;
@@ -159,6 +213,10 @@ public:
     Quatf trackingOrientation;   // orientation prior to selecting tracking
 
     float fov;
+    bool reverseFlag;
+
+    uint32 locationFilter;
+    std::string displayedSurface;
 };
 
 #endif // _CELENGINE_OBSERVER_H_
