@@ -18,6 +18,9 @@
 #include "body.h"
 #include "parser.h"
 
+class ReferenceFrame;
+class Universe;
+
 bool ParseDate(Hash* hash, const string& name, double& jd);
 
 Orbit* CreateOrbit(PlanetarySystem* system,
@@ -25,6 +28,13 @@ Orbit* CreateOrbit(PlanetarySystem* system,
                    const std::string& path,
                    bool usePlanetUnits);
 
-void FillinRotationElements(Hash* rotationData, RotationElements& re);
+RotationModel* CreateRotationModel(Hash* rotationData,
+                                   const string& path,
+                                   double syncRotationPeriod);
+
+RotationModel* CreateDefaultRotationModel(double syncRotationPeriod);
+
+ReferenceFrame* CreateReferenceFrame(const Universe& universe,
+                                     Value* frameValue);
 
 #endif // _CELENGINE_PARSEOBJECT_H_
