@@ -1,17 +1,20 @@
 // render.h
 //
-// Copyright (C) 2001, Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2001-2007, Celestia Development Team
+// Contact: Chris Laurel <claurel@gmail.com>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // as published by the Free Software Foundation; either version 2
 // of the License, or (at your option) any later version.
 
-#ifndef _RENDER_H_
-#define _RENDER_H_
+#ifndef _CELENGINE_RENDER_H_
+#define _CELENGINE_RENDER_H_
 
 #include <vector>
+#include <list>
 #include <string>
+#include <celmath/frustum.h>
 #include <celengine/observer.h>
 #include <celengine/universe.h>
 #include <celengine/selection.h>
@@ -21,25 +24,46 @@
 #include <celtxf/texturefont.h>
 
 
+class RendererWatcher;
+
+struct LightSource
+{
+    Point3d position;
+    Color color;
+    float luminosity;
+    float radius;
+};
+
+
 struct RenderListEntry
 {
+    enum RenderableType
+    {
+        RenderableStar,
+        RenderableBody,
+        RenderableCometTail,
+        RenderableBodyAxes,
+        RenderableFrameAxes,
+        RenderableSunDirection,
+        RenderableVelocityVector,
+    };
+
     const Star* star;
     Body* body;
     Point3f position;
     Vec3f sun;
     float distance;
     float radius;
+    float centerZ;
     float nearZ;
     float farZ;
     float discSizeInPixels;
     float appMag;
-    bool isCometTail;
-    int depthBucket;
-    int solarSysIndex;
+    RenderableType renderableType;
+    bool isOpaque;
+    std::vector<LightSource>* lightSourceList;
 };
 
-static const unsigned int MaxSolarSystems = 16;
-
 
 class Renderer
 {
@@ -82,8 +106,8 @@ class Renderer
         SpacecraftLabels    = 0x040,
         LocationLabels      = 0x080,
         CometLabels         = 0x100,
-	NebulaLabels        = 0x200,
-	OpenClusterLabels   = 0x400,
+        NebulaLabels        = 0x200,
+        OpenClusterLabels   = 0x400,
         I18nConstellationLabels = 0x800,
         BodyLabelMask       = (PlanetLabels | MoonLabels | AsteroidLabels | SpacecraftLabels | CometLabels),
     };
@@ -108,8 +132,9 @@ class Renderer
         ShowCometTails      = 0x8000,
         ShowMarkers         = 0x10000,
         ShowPartialTrajectories = 0x20000,
-	ShowNebulae         = 0x40000,
-	ShowOpenClusters    = 0x80000,
+        ShowNebulae         = 0x40000,
+        ShowOpenClusters    = 0x80000,
+        ShowCloudShadows    = 0x200000,
     };
 
     enum StarStyle 
@@ -120,12 +145,24 @@ class Renderer
         StarStyleCount   = 3,
     };
 
+    // constants
+    static const int DefaultRenderFlags = Renderer::ShowStars          |
+                                          Renderer::ShowPlanets        |
+                                          Renderer::ShowGalaxies       |
+                                          Renderer::ShowCloudMaps      |
+                                          Renderer::ShowAtmospheres    |
+                                          Renderer::ShowEclipseShadows |
+                                          Renderer::ShowRingShadows    |
+                                          Renderer::ShowCometTails     |
+                                          Renderer::ShowNebulae        |
+                                          Renderer::ShowOpenClusters   |
+                                          Renderer::ShowAutoMag        |
+                                          Renderer::ShowSmoothLines;
+
     int getRenderFlags() const;
     void setRenderFlags(int);
     int getLabelMode() const;
     void setLabelMode(int);
-    void addLabelledStar(Star*, const std::string&);
-    void clearLabelledStars();
     float getAmbientLightLevel() const;
     void setAmbientLightLevel(float);
     float getMinimumOrbitSize() const;
@@ -140,6 +177,8 @@ class Renderer
     void setScreenDpi(int);
     const ColorTemperatureTable* getStarColorTable() const;
     void setStarColorTable(const ColorTemperatureTable*);
+    bool getVideoSync() const;
+    void setVideoSync(bool);
 
     bool getFragmentShaderEnabled() const;
     void setFragmentShaderEnabled(bool);
@@ -150,10 +189,6 @@ class Renderer
 
     GLContext* getGLContext() { return context; }
 
-    float getSaturationMagnitude() const;
-    void setSaturationMagnitude(float);
-    float getBrightnessBias() const;
-    void setBrightnessBias(float);
     void setStarStyle(StarStyle);
     StarStyle getStarStyle() const;
     void setResolution(unsigned int resolution);
@@ -161,18 +196,49 @@ class Renderer
 
     void loadTextures(Body*);
 
-    typedef struct {
-        std::string text;
+    // Label related methods
+    static const int MaxLabelLength = 32;
+    struct Label
+    {
+        char text[MaxLabelLength];
         Color color;
         Point3f position;
-    } Label;
-
-    void addLabel(std::string, Color, const Point3f&, float depth = -1);
-    void addSortedLabel(std::string, Color, const Point3f&);
+    };
+    
+    void addLabel(const char* text, Color, const Point3f&, float depth = -1);
+    void addLabel(const std::string&, Color, const Point3f&, float depth = -1);
+    void addSortedLabel(const std::string&, Color, const Point3f&);
     void clearLabels();
+	void clearSortedLabels();
 
-    void setFont(TextureFont*);
-    TextureFont* getFont() const;
+    void invalidateOrbitCache();
+    
+    struct OrbitPathListEntry
+    {
+        float centerZ;
+        float radius;
+        Body* body;
+        const Star* star;
+        Point3f origin;
+        float opacity;
+    };        
+
+    enum FontStyle
+    {
+        FontNormal = 0,
+        FontLarge  = 1,
+        FontCount  = 2,
+    };
+    
+    void setFont(FontStyle, TextureFont*);
+    TextureFont* getFont(FontStyle) const;
+
+    bool settingsHaveChanged() const;
+    void markSettingsChanged();
+
+    void addWatcher(RendererWatcher*);
+    void removeWatcher(RendererWatcher*);
+    void notifyWatchers() const;
 
  public:
     // Internal types
@@ -215,7 +281,7 @@ class Renderer
     public:
         StarVertexBuffer(unsigned int _capacity);
         ~StarVertexBuffer();
-        void start(bool _usePoints);
+        void start();
         void render();
         void finish();
         void addStar(const Point3f&, const Color&, float);
@@ -228,27 +294,20 @@ class Renderer
         float* texCoords;
         unsigned char* colors;
         Vec3f v0, v1, v2, v3;
-        bool usePoints;
     };
 
-    struct LightSource
-    {
-        Point3d position;
-        Color color;
-        float luminosity;
-        float radius;
-    };
 
-    
     class PointStarVertexBuffer
     {
     public:
         PointStarVertexBuffer(unsigned int _capacity);
         ~PointStarVertexBuffer();
-        void start(const GLContext&);
+        void startPoints(const GLContext&);
+        void startSprites(const GLContext&);
         void render();
         void finish();
         void addStar(const Point3f& f, const Color&, float);
+		void setTexture(Texture*);
 
     private:
         struct StarVertex
@@ -263,6 +322,8 @@ class Renderer
         unsigned int nStars;
         StarVertex* vertices;
         const GLContext* context;
+        bool useSprites;
+		Texture* texture;
     };
 
  private:
@@ -309,24 +370,43 @@ class Renderer
         };
     };
 
+    enum LabelAlignment
+    {
+        AlignCenter,
+        AlignLeft,
+        AlignRight
+    };
     typedef ObjectLabel<Star>          StarLabel;
     typedef ObjectLabel<DeepSkyObject> DSOLabel;    // currently not used
+    
+    struct DepthBufferPartition
+    {
+        int index;
+        float nearZ;
+        float farZ;
+    };
 
  private:
     void setFieldOfView(float);
     void renderStars(const StarDatabase& starDB,
                      float faintestVisible,
                      const Observer& observer);
+    void renderPointStars(const StarDatabase& starDB,
+                          float faintestVisible,
+                          const Observer& observer);
     void renderDeepSkyObjects(const Universe&,
                               const Observer&,
                               float faintestMagNight);
     void renderCelestialSphere(const Observer& observer);
-    void renderPlanetarySystem(const Star& sun,
-                               const PlanetarySystem& solSystem,
-                               const Observer& observer,
-                               double now,
-                               unsigned int solarSysIndex,
-                               bool showLabels = false);
+    void buildRenderLists(const Star& sun,
+                          const PlanetarySystem* solSystem,
+                          const Observer& observer,
+                          double now,
+                          std::vector<LightSource>* lightSourceList,
+                          bool showLabels = false);
+    void addStarOrbitToRenderList(const Star& star,
+                                  const Observer& observer,
+                                  double now);
 
     void renderObject(Point3f pos,
                       float distance,
@@ -341,8 +421,8 @@ class Renderer
                       Point3f pos,
                       float distance,
                       float appMag,
-                      double now,
-                      Quatf orientation,
+                      const Observer& observer,
+                      const Quatf& cameraOrientation,
                       const vector<LightSource>& lightSources,
                       float, float);
 
@@ -354,23 +434,52 @@ class Renderer
                     double now,
                     float, float);
 
+    void renderAxes(Body& body,
+                    Point3f pos,
+                    float distance,
+                    double now,
+                    float nearPlaneDistance,
+                    float farPlaneDistance,
+                    RenderListEntry::RenderableType renderableType);
+
+    void renderSunDirection(Body& body,
+                            Point3f pos,
+                            float distance,
+                            double now,
+                            const vector<LightSource>& lightSources,
+                            float nearPlaneDistance,
+                            float farPlaneDistance);
+
+    void renderVelocityVector(Body& body,
+                              Point3f pos,
+                              float distance,
+                              double now,
+                              float nearPlaneDistance,
+                              float farPlaneDistance);
+
     void renderCometTail(const Body& body,
                          Point3f pos,
-                         float distance,
-                         float appMag,
                          double now,
-                         Quatf orientation,
                          const vector<LightSource>& lightSources,
-                         float, float);
+                         float discSizeInPixels);
 
     void renderBodyAsParticle(Point3f center,
                               float appMag,
                               float _faintestMag,
                               float discSizeInPixels,
                               Color color,
-                              const Quatf& orientation,
+                              const Quatf& cameraOrientation,
                               float renderDistance,
-                              bool useHaloes);
+                              bool useHalos);
+    void renderObjectAsPoint(Point3f center,
+                             float radius,
+                             float appMag,
+                             float _faintestMag,
+                             float discSizeInPixels,
+                             Color color,
+                             const Quatf& cameraOrientation,
+                             bool useHalos,
+                             bool emissive);
 
     void renderEllipsoidAtmosphere(const Atmosphere& atmosphere,
                                    Point3f center,
@@ -383,9 +492,16 @@ class Renderer
 
     void renderLocations(const vector<Location*>& locations,
                          const Quatf& cameraOrientation,
-                         const Point3f& position,
-                         const Quatf& orientation,
+                         const Point3d& bodyPosition,
+                         const Quatd& bodyOrientation,
                          float scale);
+                   
+    // Render an item from the render list                   
+    void renderItem(const RenderListEntry& rle,
+                    const Observer& observer,
+                    const Quatf& cameraOrientation,
+                    float nearPlaneDistance,
+                    float farPlaneDistance);
 
     bool testEclipse(const Body& receiver,
                      const Body& caster,
@@ -393,30 +509,26 @@ class Renderer
                      double now,
                      vector<EclipseShadow>& shadows);
 
-    void labelStars(const std::vector<StarLabel>& labelledStars,
-                    const StarDatabase&,
-                    const Observer&);
     void labelConstellations(const AsterismList& asterisms,
                              const Observer& observer);
     void renderParticles(const std::vector<Particle>& particles,
                          Quatf orientation);
-    void renderLabels();
+    void renderLabels(FontStyle fs, LabelAlignment la);
     std::vector<Label>::iterator renderSortedLabels(std::vector<Label>::iterator,
-                                                    float depth);
+                                                    float nearDist,
+                                                    float farDist,
+                                                    FontStyle fs);
     void renderMarkers(const MarkerList&,
                        const UniversalCoord& position,
                        const Quatf& orientation,
                        double jd);
 
-    void renderOrbit(Body*, double);
-    void renderOrbits(PlanetarySystem*, const Selection&, double,
-                      const Point3d&, const Point3d&);
-    void renderForegroundOrbits(const PlanetarySystem* system,
-                                const Point3f &center,
-                                float distance,
-                                float discSizeInPixels,
-                                const Selection& sel, 
-                                double t);
+    void renderOrbit(const OrbitPathListEntry&,
+                     double now,
+                     const Quatf& cameraOrientation,
+                     const Frustum& frustum,
+                     float nearDist,
+                     float farDist);
 
  private:
     GLContext* context;
@@ -428,7 +540,7 @@ class Renderer
     float corrFac;
     float pixelSize;
     float faintestAutoMag45deg;
-    TextureFont* font;
+    TextureFont* font[FontCount];
 
     int renderMode;
     int labelMode;
@@ -450,17 +562,18 @@ class Renderer
     std::string displayedSurface;
 
     StarVertexBuffer* starVertexBuffer;
+    PointStarVertexBuffer* pointStarVertexBuffer;
+	PointStarVertexBuffer* glareVertexBuffer;
     std::vector<RenderListEntry> renderList;
+    std::vector<DepthBufferPartition> depthPartitions;
     std::vector<Particle> glareParticles;
     std::vector<Label> labels;
     std::vector<Label> depthSortedLabels;
+    std::vector<OrbitPathListEntry> orbitPathList;
     std::vector<EclipseShadow> eclipseShadows[MaxLights];
     std::vector<const Star*> nearStars;
 
-    std::vector<LightSource> lightSourceLists[MaxSolarSystems];
-
-    std::vector<StarLabel> labelledStars;
-    std::vector<DSOLabel>  labelledDSOs;
+    std::list<std::vector<LightSource>* > lightSourceLists;
 
     double modelMatrix[16];
     double projMatrix[16];
@@ -473,22 +586,39 @@ class Renderer
     unsigned int textureResolution;
 
     DetailOptions detailOptions;
+    
+    bool useNewStarRendering;
+
+    uint32 frameCount;
 
  public:
     struct OrbitSample 
     {
         double t;
-        Point3f pos;
+        Point3d pos;
+        
+        OrbitSample(const Point3d& _pos, double _t) : t(_t), pos(_pos) { }
+        OrbitSample() { }
     };
 
- private:
+    struct OrbitSection
+    {
+        Capsuled boundingVolume;
+        uint32 firstSample;
+    };
+    
     struct CachedOrbit
     {
         Body* body;
         std::vector<OrbitSample> trajectory;
-        bool keep;
+        std::vector<OrbitSection> sections;
+        uint32 lastUsed;
     };
-    std::vector<CachedOrbit*> orbitCache;
+
+ private:
+    typedef std::map<Body*, CachedOrbit*> OrbitCache;
+    OrbitCache orbitCache;
+    uint32 lastOrbitCacheFlush;
 
     float minOrbitSize;
     float distanceLimit;
@@ -500,6 +630,51 @@ class Renderer
     SkyContourPoint* skyContour;
 
     const ColorTemperatureTable* colorTemp;
+    
+    Selection highlightObject;
+
+    bool videoSync;
+    bool settingsChanged;
+
+    std::list<RendererWatcher*> watchers;
+
+ public:
+    // Colors for all lines and labels
+    static Color StarLabelColor;
+    static Color PlanetLabelColor;
+    static Color MoonLabelColor;
+    static Color AsteroidLabelColor;
+    static Color CometLabelColor;
+    static Color SpacecraftLabelColor;
+    static Color LocationLabelColor;
+    static Color GalaxyLabelColor;
+    static Color NebulaLabelColor;
+    static Color OpenClusterLabelColor;
+    static Color ConstellationLabelColor;
+    static Color EquatorialGridLabelColor;
+
+    static Color StarOrbitColor;
+    static Color PlanetOrbitColor;
+    static Color MoonOrbitColor;
+    static Color AsteroidOrbitColor;
+    static Color CometOrbitColor;
+    static Color SpacecraftOrbitColor;
+    static Color SelectionOrbitColor;
+
+    static Color ConstellationColor;
+    static Color BoundaryColor;
+    static Color EquatorialGridColor;
 };
 
-#endif // _RENDER_H_
+
+class RendererWatcher
+{
+ public:
+    RendererWatcher() {};
+    virtual ~RendererWatcher() {};
+
+    virtual void notifyRenderSettingsChanged(const Renderer*) = 0;
+};
+
+
+#endif // _CELENGINE_RENDER_H_
