@@ -1,6 +1,6 @@
 // stardb.cpp
 //
-// Copyright (C) 2001, Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2001-2008, Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -107,7 +107,7 @@ static bool parseSimpleCatalogNumber(const string& name,
 }
 
 
-static bool parseHIPPARCOSCatalogNumber(const string& name, 
+static bool parseHIPPARCOSCatalogNumber(const string& name,
                                         uint32* catalogNumber)
 {
     return parseSimpleCatalogNumber(name,
@@ -116,7 +116,7 @@ static bool parseHIPPARCOSCatalogNumber(const string& name,
 }
 
 
-static bool parseHDCatalogNumber(const string& name, 
+static bool parseHDCatalogNumber(const string& name,
                                  uint32* catalogNumber)
 {
     return parseSimpleCatalogNumber(name,
@@ -152,7 +152,7 @@ static bool parseCelestiaCatalogNumber(const string& name,
     if (name[0] == '#')
     {
         unsigned int num;
-        if (sscanf(name.c_str(), "#%u %c", &num, &extra) == 1)
+        if (sscanf(name.c_str(), "#%u %c", &num, extra) == 1)
         {
             *catalogNumber = (uint32) num;
             return true;
@@ -215,48 +215,54 @@ Star* StarDatabase::find(const uint32 catalogNumber) const
 }
 
 
-Star* StarDatabase::find(const string& name) const
+uint32 StarDatabase::findCatalogNumberByName(const string& name) const
 {
     if (name.empty())
-        return NULL;
+        return Star::InvalidCatalogNumber;
 
-    uint32 catalogNumber = 0;
+    uint32 catalogNumber = Star::InvalidCatalogNumber;
 
     if (parseCelestiaCatalogNumber(name, &catalogNumber))
     {
-        return find(catalogNumber);
+        return catalogNumber;
     }
     else if (parseHIPPARCOSCatalogNumber(name, &catalogNumber))
     {
-        return find(catalogNumber);
+        return catalogNumber;
     }
     else if (parseTychoCatalogNumber(name, &catalogNumber))
     {
-        return find(catalogNumber);
+        return catalogNumber;
     }
     else if (parseHDCatalogNumber(name, &catalogNumber))
     {
-        return searchCrossIndex(HenryDraper, catalogNumber);
+        return searchCrossIndexForCatalogNumber(HenryDraper, catalogNumber);
     }
     else if (parseSimpleCatalogNumber(name, SAOCatalogPrefix,
                                       &catalogNumber))
     {
-        return searchCrossIndex(SAO, catalogNumber);
+        return searchCrossIndexForCatalogNumber(SAO, catalogNumber);
     }
     else
     {
         if (namesDB != NULL)
-        {
-            uint32 catalogNumber   = namesDB->findCatalogNumberByName(name);
-            if (catalogNumber != Star::InvalidCatalogNumber)
-                return find(catalogNumber);
-        }
-
-        return NULL;
+            return namesDB->findCatalogNumberByName(name);
+        else
+            return Star::InvalidCatalogNumber;
     }
 }
 
 
+Star* StarDatabase::find(const string& name) const
+{
+    uint32 catalogNumber = findCatalogNumberByName(name);
+    if (catalogNumber != Star::InvalidCatalogNumber)
+        return find(catalogNumber);
+    else
+        return NULL;
+}
+
+
 uint32 StarDatabase::crossIndex(const Catalog catalog, const uint32 celCatalogNumber) const
 {
     if (static_cast<uint32>(catalog) >= crossIndexes.size())
@@ -278,14 +284,16 @@ uint32 StarDatabase::crossIndex(const Catalog catalog, const uint32 celCatalogNu
 }
 
 
-Star* StarDatabase::searchCrossIndex(const Catalog catalog, const uint32 number) const
+// Return the Celestia catalog number for the star with a specified number
+// in a cross index.
+uint32 StarDatabase::searchCrossIndexForCatalogNumber(const Catalog catalog, const uint32 number) const
 {
     if (static_cast<unsigned int>(catalog) >= crossIndexes.size())
-        return NULL;
+        return Star::InvalidCatalogNumber;
 
     CrossIndex* xindex = crossIndexes[catalog];
     if (xindex == NULL)
-        return NULL;
+        return Star::InvalidCatalogNumber;
 
     CrossIndexEntry xindexEnt;
     xindexEnt.catalogNumber = number;
@@ -293,9 +301,19 @@ Star* StarDatabase::searchCrossIndex(const Catalog catalog, const uint32 number)
     CrossIndex::iterator iter = lower_bound(xindex->begin(), xindex->end(),
                                             xindexEnt);
     if (iter == xindex->end() || iter->catalogNumber != number)
-        return NULL;
+        return Star::InvalidCatalogNumber;
     else
-        return find(iter->celCatalogNumber);
+        return iter->celCatalogNumber;
+}
+
+
+Star* StarDatabase::searchCrossIndex(const Catalog catalog, const uint32 number) const
+{
+    uint32 celCatalogNumber = searchCrossIndexForCatalogNumber(catalog, number);
+    if (celCatalogNumber != Star::InvalidCatalogNumber)
+        return find(celCatalogNumber);
+    else
+        return NULL;
 }
 
 
@@ -311,6 +329,30 @@ vector<string> StarDatabase::getCompletion(const string& name) const
 }
 
 
+static void catalogNumberToString(uint32 catalogNumber, char* buf, unsigned int bufSize)
+{
+    // Just return an empty string if there's any chance that the buffer is too small
+    if (bufSize < 20 && bufSize > 0)
+    {
+        buf[0] = '\0';
+    }
+
+    if (catalogNumber < 1000000)
+    {
+        sprintf(buf, "HIP %d", catalogNumber);
+    }
+    else
+    {
+        uint32 tyc3 = catalogNumber / 1000000000;
+        catalogNumber -= tyc3 * 1000000000;
+        uint32 tyc2 = catalogNumber / 10000;
+        catalogNumber -= tyc2 * 10000;
+        uint32 tyc1 = catalogNumber;
+        sprintf(buf, "TYC %d-%d-%d", tyc1, tyc2, tyc3);
+    }
+}
+
+
 // Return the name for the star with specified catalog number.  The returned
 // string will be:
 //      the common name if it exists, otherwise
@@ -344,22 +386,32 @@ string StarDatabase::getStarName(const Star& star) const
       sprintf(buf, "HD %d", star.getCatalogNumber(Star::HDCatalog));
       else
     */
+    catalogNumberToString(catalogNumber, buf, sizeof(buf));
+
+    return string(buf);
+}
+
+// A less convenient version of getStarName that writes to a char
+// array instead of a string. The advantage is that no memory allocation
+// will every occur.
+void StarDatabase::getStarName(const Star& star, char* nameBuffer, unsigned int bufferSize) const
+{
+    assert(bufferSize != 0);
+
+    uint32 catalogNumber = star.getCatalogNumber();
+
+    if (namesDB != NULL)
     {
-        if (catalogNumber < 1000000)
+        StarNameDatabase::NumberIndex::const_iterator iter = namesDB->getFirstNameIter(catalogNumber);
+        if (iter != namesDB->getFinalNameIter() && iter->first == catalogNumber)
         {
-            sprintf(buf, "HIP %d", catalogNumber);
-        }
-        else
-        {
-            uint32 tyc3 = catalogNumber / 1000000000;
-            catalogNumber -= tyc3 * 1000000000;
-            uint32 tyc2 = catalogNumber / 10000;
-            catalogNumber -= tyc2 * 10000;
-            uint32 tyc1 = catalogNumber;
-            sprintf(buf, "TYC %d-%d-%d", tyc1, tyc2, tyc3);
+            strncpy(nameBuffer, iter->second.c_str(), bufferSize);
+            nameBuffer[bufferSize - 1] = '\0';
+            return;
         }
     }
-    return string(buf);
+
+    catalogNumberToString(catalogNumber, nameBuffer, bufferSize);
 }
 
 
@@ -378,7 +430,7 @@ string StarDatabase::getStarNameList(const Star& star, const unsigned int maxNam
         if (count != 0)
             starNames   += " / ";
 
-        starNames   += ReplaceGreekLetterAbbr(_(iter->second.c_str()));
+        starNames   += ReplaceGreekLetterAbbr(iter->second.c_str());
         ++iter;
         ++count;
     }
@@ -458,7 +510,7 @@ void StarDatabase::findVisibleStars(StarHandler& starHandler,
         planeNormals[i] = planeNormals[i] * rot;
         frustumPlanes[i] = Planef(planeNormals[i], position);
     }
-    
+
     octreeRoot->processVisibleObjects(starHandler,
                                       position,
                                       frustumPlanes,
@@ -572,7 +624,7 @@ bool StarDatabase::loadOldFormatBinary(istream& in)
         Star* newStars = new Star[requiredCapacity];
         if (newStars == NULL)
             return false;
-        
+
         if (stars != NULL)
         {
             copy(stars, stars + nStars, newStars);
@@ -583,7 +635,7 @@ bool StarDatabase::loadOldFormatBinary(istream& in)
 
         capacity = requiredCapacity;
     }
-    
+
     uint32 throwOut = 0;
     uint32 fixUp = 0;
     unsigned int totalStars = nStars + nStarsInFile;
@@ -633,7 +685,7 @@ bool StarDatabase::loadOldFormatBinary(istream& in)
 	star->setAbsoluteMagnitude((float) (appMag / 256.0 + 5 -
 					    5 * log10(distance / 3.26)));
 
-        
+
         StarDetails* details = NULL;
         StellarClass sc;
         if (sc.unpack(spectralType))
@@ -644,7 +696,7 @@ bool StarDatabase::loadOldFormatBinary(istream& in)
             cerr << _("Bad spectral type in star database, star #\n");
             return false;
         }
-        
+
         star->setDetails(details);
         star->setCatalogNumber(catNo);
 
@@ -706,7 +758,7 @@ bool StarDatabase::loadBinary(istream& in)
         Star* newStars = new Star[requiredCapacity];
         if (newStars == NULL)
             return false;
-        
+
         if (stars != NULL)
         {
             copy(stars, stars + nStars, newStars);
@@ -717,7 +769,7 @@ bool StarDatabase::loadBinary(istream& in)
 
         capacity = requiredCapacity;
     }
-    
+
     unsigned int totalStars = nStars + nStarsInFile;
 
     while (((unsigned int) nStars) < totalStars)
@@ -805,8 +857,12 @@ void StarDatabase::finish()
         assert(star != NULL);
         assert(barycenter != NULL);
         if (star != NULL && barycenter != NULL)
+        {
             star->setOrbitBarycenter(barycenter);
+            barycenter->addOrbitingStar(star);
+        }
     }
+
     barycenters.clear();
 }
 
@@ -825,16 +881,16 @@ static void stcError(const Tokenizer& tok,
 
 
 Star* StarDatabase::createStar(const uint32 catalogNumber,
-                         Hash* starData,
-                         const string& path,
-                         bool isBarycenter)
+                               Hash* starData,
+                               const string& path,
+                               bool isBarycenter)
 {
     StarDetails* details = NULL;
     string spectralType;
 
     // Get the magnitude and spectral type; if the star is actually
     // a barycenter placeholder, these fields are ignored.
-    double magnitude = 0.0;
+    /*double magnitude = 0.0;   Unused*/
     if (isBarycenter)
     {
         details = StarDetails::GetBarycenterDetails();
@@ -860,9 +916,8 @@ Star* StarDatabase::createStar(const uint32 catalogNumber,
     bool hasTexture = starData->getString("Texture", textureName);
     bool hasModel = starData->getString("Mesh", modelName);
 
-    RotationElements re = details->getRotationElements();
-    FillinRotationElements(starData, re);
-    bool hasRotationElements = !(re == details->getRotationElements());
+    RotationModel* rm = CreateRotationModel(starData, path, 1.0);
+    bool hasRotationModel = (rm != NULL);
 
     Vec3d semiAxes;
     bool hasSemiAxes = starData->getVector("SemiAxes", semiAxes);
@@ -879,15 +934,11 @@ Star* StarDatabase::createStar(const uint32 catalogNumber,
         orbit != NULL   ||
         hasSemiAxes     ||
         hasRadius       ||
-        hasRotationElements)
+        hasRotationModel)
     {
         // If the star definition has extended information, clone the
         // star details so we can customize it without affecting other
         // stars of the same spectral type.
-        // TODO: Need better management of star details objects.  The
-        // standard ones should be persistent, but the custom ones should
-        // probably be destroyed in the star destructor.  Reference counting
-        // is probably the best strategy.
         details = new StarDetails(*details);
 
         if (hasTexture)
@@ -917,10 +968,22 @@ Star* StarDatabase::createStar(const uint32 catalogNumber,
             details->setOrbit(orbit);
 
             // See if a barycenter was specified as well
+            uint32 barycenterCatNo = Star::InvalidCatalogNumber;
+            bool barycenterDefined = false;
+
             string barycenterName;
             if (starData->getString("OrbitBarycenter", barycenterName))
             {
-                uint32 barycenterCatNo   = namesDB->findCatalogNumberByName(barycenterName);
+                barycenterCatNo   = findCatalogNumberByName(barycenterName);
+                barycenterDefined = true;
+            }
+            else if (starData->getNumber("OrbitBarycenter", barycenterCatNo))
+            {
+                barycenterDefined = true;
+            }
+
+            if (barycenterDefined)
+            {
                 if (barycenterCatNo != Star::InvalidCatalogNumber)
                 {
                     // We can't actually resolve the barycenter catalog number
@@ -943,10 +1006,10 @@ Star* StarDatabase::createStar(const uint32 catalogNumber,
                     if (nStars > 0)
                     {
                         uint32 starIndex = nStars;
-                        do 
+                        do
                         {
                             starIndex--;
-                            if (stars[starIndex].getCatalogNumber() == 
+                            if (stars[starIndex].getCatalogNumber() ==
                                 barycenterCatNo)
                             {
                                 hasBarycenter = true;
@@ -965,8 +1028,8 @@ Star* StarDatabase::createStar(const uint32 catalogNumber,
             }
         }
 
-        if (hasRotationElements)
-            details->setRotationElements(re);
+        if (hasRotationModel)
+            details->setRotationModel(rm);
     }
 
     Star* star = new Star();
@@ -1026,12 +1089,23 @@ Star* StarDatabase::createStar(const uint32 catalogNumber,
         {
             if (!starData->getNumber("AppMag", magnitude))
             {
-                cerr << _("Invalid star: missing magnitude.\n");
-                magnitude = 30.0;
+                clog << _("Invalid star: missing magnitude.\n");
+                delete star;
+                return NULL;
             }
             else
             {
                 float distance = star->getPosition().distanceFromOrigin();
+
+                // We can't compute the intrinsic brightness of the star from
+                // the apparent magnitude if the star is within a few AU of the
+                // origin.
+                if (distance < 1e-5f)
+                {
+                    clog << _("Invalid star: absolute (not apparent) magnitude must be specified for star near origin\n");
+                    delete star;
+                    return NULL;
+                }
                 magnitude = astro::appToAbsMag((float) magnitude, distance);
             }
         }
@@ -1090,7 +1164,7 @@ bool StarDatabase::load(istream& in, const string& resourcePath)
             objName    = tokenizer.getStringValue();
             tokenizer.nextToken();
         }
-        
+
         tokenizer.pushBack();
 
         Value* starDataValue = parser.readValue();
@@ -1103,11 +1177,14 @@ bool StarDatabase::load(istream& in, const string& resourcePath)
         if (starDataValue->getType() != Value::HashType)
         {
             DPRINTF(0, "Bad star definition.\n");
+            delete starDataValue;
             return false;
         }
         Hash* starData = starDataValue->getHash();
 
         Star* star   = createStar(catalogNumber, starData, resourcePath, !isStar);
+        delete starDataValue;
+
         if (star != NULL)
         {
             // Ensure that the star array is large enough
@@ -1138,6 +1215,7 @@ bool StarDatabase::load(istream& in, const string& resourcePath)
                 stars = newStars;
             }
             stars[nStars++] = *star;
+            delete star;
 
             if (namesDB != NULL && !objName.empty())
             {
@@ -1148,7 +1226,7 @@ bool StarDatabase::load(istream& in, const string& resourcePath)
                 // Iterate through the string for names delimited
                 // by ':', and insert them into the star database.
                 // Note that db->add() will skip empty namesDB.
-                string::size_type startPos = 0; 
+                string::size_type startPos = 0;
                 while (startPos != string::npos)
                 {
                     string::size_type next    = objName.find(':', startPos);
@@ -1158,6 +1236,8 @@ bool StarDatabase::load(istream& in, const string& resourcePath)
                         length = next - startPos;
                         ++next;
                     }
+                    if (catalogNumber == 0 && objName.substr(startPos, length) == "Sun" && string("Sun") != _("Sun"))
+                        namesDB->add(0, _("Sun"));
                     namesDB->add(catalogNumber, objName.substr(startPos, length));
                     startPos = next;
                 }
