@@ -1,5 +1,5 @@
 // universe.cpp
-// 
+//
 // Copyright (C) 2001, Chris Laurel <claurel@shatters.net>
 //
 // A container for catalogs of galaxies, stars, and planets.
@@ -10,13 +10,13 @@
 // of the License, or (at your option) any later version.
 
 #include <celmath/mathlib.h>
-#include <celmath/vecmath.h>
 #include <celmath/intersect.h>
 #include <celutil/utf8.h>
 #include <cassert>
 #include "astro.h"
 #include "3dsmesh.h"
 #include "meshmanager.h"
+#include "render.h"
 #include "universe.h"
 
 #define ANGULAR_RES 3.5e-6
@@ -26,8 +26,8 @@ using namespace std;
 
 Universe::Universe() :
     starCatalog(NULL),
-    solarSystemCatalog(NULL),
     dsoCatalog(NULL),
+    solarSystemCatalog(NULL),
     asterisms(NULL),
     /*boundaries(NULL),*/
     markers(NULL)
@@ -169,7 +169,8 @@ void Universe::markObject(const Selection& sel,
                           float size,
                           Color color,
                           Marker::Symbol symbol,
-                          int priority)
+                          int priority,
+                          string label)
 {
     for (MarkerList::iterator iter = markers->begin();
          iter != markers->end(); iter++)
@@ -196,6 +197,7 @@ void Universe::markObject(const Selection& sel,
     marker.setSymbol(symbol);
     marker.setSize(size);
     marker.setPriority(priority);
+    marker.setLabel(label);
     markers->insert(markers->end(), marker);
 }
 
@@ -237,7 +239,7 @@ bool Universe::isMarked(const Selection& sel, int priority) const
 class ClosestStarFinder : public StarHandler
 {
 public:
-    ClosestStarFinder(float _maxDistance);
+    ClosestStarFinder(float _maxDistance, const Universe* _universe);
     ~ClosestStarFinder() {};
     void process(const Star& star, float distance, float appMag);
 
@@ -245,19 +247,29 @@ public:
     float maxDistance;
     float closestDistance;
     Star* closestStar;
+    const Universe* universe;
+    bool withPlanets;
 };
 
-ClosestStarFinder::ClosestStarFinder(float _maxDistance) :
-    maxDistance(_maxDistance), closestDistance(_maxDistance), closestStar(NULL)
+ClosestStarFinder::ClosestStarFinder(float _maxDistance,
+                                     const Universe* _universe) :
+    maxDistance(_maxDistance),
+    closestDistance(_maxDistance),
+    closestStar(NULL),
+    universe(_universe),
+    withPlanets(false)
 {
 }
 
-void ClosestStarFinder::process(const Star& star, float distance, float appMag)
+void ClosestStarFinder::process(const Star& star, float distance, float)
 {
     if (distance < closestDistance)
     {
-        closestStar = const_cast<Star*>(&star);
-        closestDistance = distance;
+        if (!withPlanets || universe->getSolarSystem(&star))
+        {
+            closestStar = const_cast<Star*>(&star);
+            closestDistance = distance;
+        }
     }
 }
 
@@ -281,7 +293,7 @@ NearStarFinder::NearStarFinder(float _maxDistance,
 {
 }
 
-void NearStarFinder::process(const Star& star, float distance, float appMag)
+void NearStarFinder::process(const Star& star, float distance, float)
 {
     if (distance < maxDistance)
         nearStars.push_back(&star);
@@ -317,8 +329,8 @@ static bool ApproxPlanetPickTraversal(Body* body, void* info)
     // the planet rather than one of its satellites.
     float appOrbitRadius = (float) (body->getOrbit()->getBoundingRadius() /
                                     distance);
-    
-    if ((pickInfo->atanTolerance > ANGULAR_RES ? pickInfo->atanTolerance: 
+
+    if ((pickInfo->atanTolerance > ANGULAR_RES ? pickInfo->atanTolerance:
         ANGULAR_RES) > appOrbitRadius)
     {
         return true;
@@ -330,7 +342,7 @@ static bool ApproxPlanetPickTraversal(Body* body, void* info)
 
     if (sinAngle2 <= pickInfo->sinAngle2Closest)
     {
-        pickInfo->sinAngle2Closest = sinAngle2 > ANGULAR_RES ? sinAngle2 : 
+        pickInfo->sinAngle2Closest = sinAngle2 > ANGULAR_RES ? sinAngle2 :
 	                                         ANGULAR_RES;
         pickInfo->closestBody = body;
         pickInfo->closestApproxDistance = distance;
@@ -379,7 +391,7 @@ static bool ExactPlanetPickTraversal(Body* body, void* info)
             // Transform rotate the pick ray into object coordinates
             Quatf qf = body->getOrientation();
             Quatd qd(qf.w, qf.x, qf.y, qf.z);
-            Mat3d m = conjugate(qd * body->getEclipticalToGeographic(pickInfo->jd)).toMatrix3();
+            Mat3d m = conjugate(qd * body->getEclipticalToBodyFixed(pickInfo->jd)).toMatrix3();
             Ray3d r(Point3d(0, 0, 0) + (pickInfo->pickRay.origin - bpos),
                     pickInfo->pickRay.direction);
             r = r * m;
@@ -409,7 +421,7 @@ static bool ExactPlanetPickTraversal(Body* body, void* info)
         double sinAngle2 = sqrt(bodyMiss * bodyMiss)/2.0;
 
 
-        if (sinAngle2 < sin(PI/4.0) && distance > 0.0 && 
+        if (sinAngle2 < sin(PI/4.0) && distance > 0.0 &&
             distance  <= pickInfo->closestDistance)
         {
             pickInfo->closestDistance = distance;
@@ -425,10 +437,10 @@ Selection Universe::pickPlanet(SolarSystem& solarSystem,
                                const UniversalCoord& origin,
                                const Vec3f& direction,
                                double when,
-                               float faintestMag,
+                               float /*faintestMag*/,
                                float tolerance)
 {
-    double sinTol2 = (sin(tolerance/2.0) >  ANGULAR_RES ? 
+    double sinTol2 = (sin(tolerance/2.0) >  ANGULAR_RES ?
 	              sin(tolerance/2.0) : ANGULAR_RES);
     PlanetPickInfo pickInfo;
 
@@ -444,7 +456,7 @@ Selection Universe::pickPlanet(SolarSystem& solarSystem,
 
     pickInfo.pickRay = Ray3d(astrocentricOrigin,
                              Vec3d(direction.x, direction.y, direction.z));
-                             
+
     pickInfo.sinAngle2Closest = 1.0;
     pickInfo.closestDistance = 1.0e50;
     pickInfo.closestApproxDistance = 1.0e50;
@@ -464,22 +476,22 @@ Selection Universe::pickPlanet(SolarSystem& solarSystem,
         Body* closestBody = pickInfo.closestBody;
 
 	// Check if there is a satellite in front of the primary body that is
-	// sufficiently close to the pickRay 
+	// sufficiently close to the pickRay
 
 	solarSystem.getPlanets()->traverse(ApproxPlanetPickTraversal,
                                            (void*) &pickInfo);
         if (pickInfo.closestBody == closestBody)
-	  return  Selection(closestBody); 
+	  return  Selection(closestBody);
         // Nothing else around, select the body and return
 
         // Are we close enough to the satellite and is it in front of the body?
-	if ((pickInfo.sinAngle2Closest <= sinTol2) && 
-            (pickInfo.closestDistance > pickInfo.closestApproxDistance)) 
+	if ((pickInfo.sinAngle2Closest <= sinTol2) &&
+            (pickInfo.closestDistance > pickInfo.closestApproxDistance))
         return Selection(pickInfo.closestBody);
             // Yes, select the satellite
 	else
-	    return  Selection(closestBody); 
-           //  No, select the primary body 
+	    return  Selection(closestBody);
+           //  No, select the primary body
     }
 
     // If no planet was intersected by the pick ray, choose the planet|moon
@@ -522,13 +534,13 @@ StarPicker::StarPicker(const Point3f& _pickOrigin,
     pickedStar(NULL),
     pickOrigin(_pickOrigin),
     pickRay(_pickRay),
-    sinAngle2Closest(sin(angle/2.0) > ANGULAR_RES ? sin(angle/2.0) : 
+    sinAngle2Closest(sin(angle/2.0) > ANGULAR_RES ? sin(angle/2.0) :
                                                     ANGULAR_RES ),
     when(_when)
 {
 }
 
-void StarPicker::process(const Star& star, float distance, float appMag)
+void StarPicker::process(const Star& star, float, float)
 {
     Vec3f relativeStarPos = star.getPosition() - pickOrigin;
     Vec3f starDir = relativeStarPos;
@@ -560,13 +572,15 @@ void StarPicker::process(const Star& star, float distance, float appMag)
     }
 
     Vec3f starMiss = starDir - pickRay;
-    Vec3d sMd = Vec3d(starMiss.x, starMiss.y, starMiss.z); 
+    Vec3d sMd = Vec3d(starMiss.x, starMiss.y, starMiss.z);
     sinAngle2 = sqrt(sMd * sMd)/2.0;
 
     if (sinAngle2 <= sinAngle2Closest)
     {
         sinAngle2Closest = sinAngle2 > ANGULAR_RES ? sinAngle2 : ANGULAR_RES;
         pickedStar = &star;
+        if (pickedStar->getOrbitBarycenter() != NULL)
+            pickedStar = pickedStar->getOrbitBarycenter();
     }
 }
 
@@ -604,19 +618,19 @@ CloseStarPicker::CloseStarPicker(const UniversalCoord& pos,
     maxDistance(_maxDistance),
     closestStar(NULL),
     closestDistance(0.0f),
-    sinAngle2Closest(sin(angle/2.0) > ANGULAR_RES ? sin(angle/2.0) : 
+    sinAngle2Closest(sin(angle/2.0) > ANGULAR_RES ? sin(angle/2.0) :
                                       ANGULAR_RES )
 {
 }
 
 void CloseStarPicker::process(const Star& star,
                               float lowPrecDistance,
-                              float appMag)
+                              float)
 {
     if (lowPrecDistance > maxDistance)
         return;
 
-    Vec3d hPos = (star.getPosition(now) - pickOrigin) * 
+    Vec3d hPos = (star.getPosition(now) - pickOrigin) *
         astro::microLightYearsToKilometers(1.0);
     Vec3f starDir((float) hPos.x, (float) hPos.y, (float) hPos.z);
     float distance = 0.0f;
@@ -631,7 +645,7 @@ void CloseStarPicker::process(const Star& star,
             {
                 closestStar = &star;
                 closestDistance = starDir.length();
-                sinAngle2Closest = ANGULAR_RES; 
+                sinAngle2Closest = ANGULAR_RES;
                 // An exact hit--set the angle to "zero"
             }
         }
@@ -642,8 +656,8 @@ void CloseStarPicker::process(const Star& star,
         float distance = starDir.length();
         starDir.normalize();
         Vec3f starMiss = starDir - pickDir;
-        Vec3d sMd = Vec3d(starMiss.x, starMiss.y, starMiss.z ); 
-    
+        Vec3d sMd = Vec3d(starMiss.x, starMiss.y, starMiss.z );
+
         double sinAngle2 = sqrt(sMd * sMd)/2.0;
 
         if (sinAngle2 <= sinAngle2Closest &&
@@ -685,7 +699,7 @@ Selection Universe::pickStar(const UniversalCoord& origin,
     Quatf rotation;
     Vec3f n(0, 0, -1);
     Vec3f Missf = n - direction;
-    Vec3d Miss = Vec3d(Missf.x, Missf.y, Missf.z); 
+    Vec3d Miss = Vec3d(Missf.x, Missf.y, Missf.z);
     double sinAngle2 = sqrt(Miss * Miss)/2.0;
 
     if (sinAngle2 <= ANGULAR_RES)
@@ -718,38 +732,42 @@ Selection Universe::pickStar(const UniversalCoord& origin,
 class DSOPicker : public DSOHandler
 {
 public:
-    DSOPicker(const Point3d&, const Vec3d&, float);
+    DSOPicker(const Point3d&, const Vec3d&, int, float);
     ~DSOPicker() {};
 
     void process(DeepSkyObject* const &, double, float);
 
 public:
-    const DeepSkyObject* pickedDSO;
     Point3d pickOrigin;
     Vec3d   pickDir;
+    int     renderFlags;
+
+    const DeepSkyObject* pickedDSO;
     double  sinAngle2Closest;
 };
 
 
 DSOPicker::DSOPicker(const Point3d& pickOrigin,
                      const Vec3d&   pickDir,
+                     int   renderFlags,
                      float angle) :
-    pickedDSO       (NULL),
     pickOrigin      (pickOrigin),
     pickDir         (pickDir),
+    renderFlags     (renderFlags),
+    pickedDSO       (NULL),
     sinAngle2Closest(sin(angle/2.0) > ANGULAR_RES ? sin(angle/2.0) :
                                                     ANGULAR_RES )
 {
 }
 
 
-void DSOPicker::process(DeepSkyObject* const & dso, double distance, float appMag)
+void DSOPicker::process(DeepSkyObject* const & dso, double, float)
 {
+    if (!(dso->getRenderMask() & renderFlags))
+        return;
+
     Vec3d relativeDSOPos = dso->getPosition() - pickOrigin;
     Vec3d dsoDir = relativeDSOPos;
-    dsoDir.normalize();
-
-    double sinAngle2 = 0.0;
 
     double distance2;
     if (testIntersection(Ray3d(Point3d(0.0, 0.0, 0.0), pickDir),
@@ -759,11 +777,11 @@ void DSOPicker::process(DeepSkyObject* const & dso, double distance, float appMa
         dsoDir         = Vec3d(dsoPos.x * 1.0e-6 - pickOrigin.x,
                                dsoPos.y * 1.0e-6 - pickOrigin.y,
                                dsoPos.z * 1.0e-6 - pickOrigin.z);
-        dsoDir.normalize();
     }
+    dsoDir.normalize();
 
     Vec3d dsoMissd   = dsoDir - Vec3d(pickDir.x, pickDir.y, pickDir.z);
-    sinAngle2        = sqrt(dsoMissd * dsoMissd)/2.0;
+    double sinAngle2 = sqrt(dsoMissd * dsoMissd)/2.0;
 
     if (sinAngle2 <= sinAngle2Closest)
     {
@@ -778,53 +796,56 @@ class CloseDSOPicker : public DSOHandler
 public:
     CloseDSOPicker(const Point3d& pos,
                    const Vec3d& dir,
+                   int    renderFlags,
                    double maxDistance,
-                   float angle);
+                   float);
     ~CloseDSOPicker() {};
 
     void process(DeepSkyObject* const & dso, double distance, float appMag);
 
 public:
-    Point3d              pickOrigin;
-    Vec3d                pickDir;
-    double maxDistance;
+    Point3d pickOrigin;
+    Vec3d   pickDir;
+    int     renderFlags;
+    double  maxDistance;
+
     const DeepSkyObject* closestDSO;
-    double closestDistance;
+    double largestCosAngle;
 };
 
 
 CloseDSOPicker::CloseDSOPicker(const Point3d& pos,
                                const Vec3d& dir,
-                               double                maxDistance,
-                               float angle) :
-    pickOrigin      (pos),
-    pickDir         (dir),
-    maxDistance     (maxDistance),
-    closestDSO      (NULL),
-    closestDistance(1.0e32)
+                               int    renderFlags,
+                               double maxDistance,
+                               float) :
+    pickOrigin     (pos),
+    pickDir        (dir),
+    renderFlags    (renderFlags),
+    maxDistance    (maxDistance),
+    closestDSO     (NULL),
+    largestCosAngle(-2.0)
 {
 }
 
 
 void CloseDSOPicker::process(DeepSkyObject* const & dso,
                              double distance,
-                             float appMag)
+                             float)
 {
-
-    if (distance > maxDistance)
+    if (distance > maxDistance || !(dso->getRenderMask() & renderFlags))
         return;
 
-    double  distance2  = 0.0;
-    if (testIntersection(Ray3d(pickOrigin, pickDir),
-                         Sphered(dso->getPosition(), (double) dso->getRadius()),
-                         distance2))
+    double  distanceToPicker       = 0.0;
+    double  cosAngleToBoundCenter  = 0.0;
+    if (dso->pick(Ray3d(pickOrigin, pickDir), distanceToPicker, cosAngleToBoundCenter))
     {
         // Don't select the object the observer is currently in:
         if (pickOrigin.distanceTo(dso->getPosition()) > dso->getRadius() &&
-            distance2 < closestDistance)
+            cosAngleToBoundCenter > largestCosAngle)
         {
-            closestDSO = dso;
-            closestDistance  = distance2;
+            closestDSO      = dso;
+            largestCosAngle = cosAngleToBoundCenter;
         }
     }
 }
@@ -832,6 +853,7 @@ void CloseDSOPicker::process(DeepSkyObject* const & dso,
 
 Selection Universe::pickDeepSkyObject(const UniversalCoord& origin,
                                       const Vec3f& direction,
+                                      int   renderFlags,
                                       float faintestMag,
                                       float tolerance)
 {
@@ -842,7 +864,7 @@ Selection Universe::pickDeepSkyObject(const UniversalCoord& origin,
 
     Vec3d dir   = Vec3d(direction.x, direction.y, direction.z);
 
-    CloseDSOPicker closePicker(orig, dir, 1e9, tolerance);
+    CloseDSOPicker closePicker(orig, dir, renderFlags, 1e9, tolerance);
 
     dsoCatalog->findCloseDSOs(closePicker, orig, 1e9);
     if (closePicker.closestDSO != NULL)
@@ -870,7 +892,7 @@ Selection Universe::pickDeepSkyObject(const UniversalCoord& origin,
         rotation.setAxisAngle(axis, (float) (2.0 * asin(sinAngle2)));
     }
 
-    DSOPicker picker(orig, dir, tolerance);
+    DSOPicker picker(orig, dir, renderFlags, tolerance);
     dsoCatalog->findVisibleDSOs(picker,
                                 orig,
                                 rotation,
@@ -887,29 +909,33 @@ Selection Universe::pickDeepSkyObject(const UniversalCoord& origin,
 Selection Universe::pick(const UniversalCoord& origin,
                          const Vec3f& direction,
                          double when,
-                         float faintestMag,
-                         float tolerance)
+                         int    renderFlags,
+                         float  faintestMag,
+                         float  tolerance)
 {
     Selection sel;
 
-    closeStars.clear();
-    getNearStars(origin, 1.0f, closeStars);
-    for (vector<const Star*>::const_iterator iter = closeStars.begin();
-         iter != closeStars.end(); iter++)
+    if (renderFlags & Renderer::ShowPlanets)
     {
-        SolarSystem* solarSystem = getSolarSystem(*iter);
-        if (solarSystem != NULL)
+        closeStars.clear();
+        getNearStars(origin, 1.0f, closeStars);
+        for (vector<const Star*>::const_iterator iter = closeStars.begin();
+            iter != closeStars.end(); iter++)
         {
-            sel = pickPlanet(*solarSystem,
-                             origin, direction,
-                             when,
-                             faintestMag,
-                             tolerance);
-            if (!sel.empty())
-                break;
+            SolarSystem* solarSystem = getSolarSystem(*iter);
+            if (solarSystem != NULL)
+            {
+                sel = pickPlanet(*solarSystem,
+                                origin, direction,
+                                when,
+                                faintestMag,
+                                tolerance);
+                if (!sel.empty())
+                    break;
+            }
         }
     }
-    
+
 #if 0
     SolarSystem* closestSolarSystem = getNearestSolarSystem(origin);
     if (closestSolarSystem != NULL)
@@ -922,11 +948,11 @@ Selection Universe::pick(const UniversalCoord& origin,
     }
 #endif
 
-    if (sel.empty())
+    if (sel.empty() && (renderFlags & Renderer::ShowStars))
         sel = pickStar(origin, direction, when, faintestMag, tolerance);
 
     if (sel.empty())
-        sel = pickDeepSkyObject(origin, direction, faintestMag, tolerance);
+        sel = pickDeepSkyObject(origin, direction, renderFlags, faintestMag, tolerance);
 
     return sel;
 }
@@ -1043,7 +1069,7 @@ Selection Universe::findObjectInContext(const Selection& sel,
 Selection Universe::find(const string& s,
                          Selection* contexts,
                          int nContexts,
-                         bool i18n)
+                         bool i18n) const
 {
     if (starCatalog != NULL)
     {
@@ -1079,8 +1105,8 @@ Selection Universe::find(const string& s,
 Selection Universe::findPath(const string& s,
                              Selection contexts[],
                              int nContexts,
-                             bool i18n)
-{ 
+                             bool i18n) const
+{
     string::size_type pos = s.find('/', 0);
 
     // No delimiter found--just do a normal find.
@@ -1117,6 +1143,7 @@ vector<string> Universe::getCompletion(const string& s,
                                                  bool withLocations)
 {
     vector<string> completion;
+    int s_length = UTF8Length(s);
 
     // Solar bodies first:
     for (int i = 0; i < nContexts; i++)
@@ -1129,7 +1156,7 @@ vector<string> Universe::getCompletion(const string& s,
                 for (vector<Location*>::const_iterator iter = locations->begin();
                      iter != locations->end(); iter++)
                 {
-                    if (!UTF8StringCompare(s, (*iter)->getName(true), s.length()))
+                    if (!UTF8StringCompare(s, (*iter)->getName(true), s_length))
                         completion.push_back((*iter)->getName(true));
                 }
             }
@@ -1234,7 +1261,8 @@ SolarSystem* Universe::getNearestSolarSystem(const UniversalCoord& position) con
 {
     Point3f pos = (Point3f) position;
     Point3f lyPos(pos.x * 1.0e-6f, pos.y * 1.0e-6f, pos.z * 1.0e-6f);
-    ClosestStarFinder closestFinder(1.0f);
+    ClosestStarFinder closestFinder(1.0f, this);
+    closestFinder.withPlanets = true;
     starCatalog->findCloseStars(closestFinder, lyPos, 1.0f);
     return getSolarSystem(closestFinder.closestStar);
 }
