@@ -1,5 +1,5 @@
 // universe.h
-// 
+//
 // Copyright (C) 2001, Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
@@ -48,28 +48,30 @@ class Universe
     Selection pick(const UniversalCoord& origin,
                    const Vec3f& direction,
                    double when,
+                   int   renderFlags,
                    float faintestMag,
                    float tolerance = 0.0f);
 
-    Selection pickStar(const UniversalCoord&,
-                       const Vec3f&,
+    Selection pickStar(const UniversalCoord& origin,
+                       const Vec3f& direction,
                        double when,
                        float faintest,
                        float tolerance = 0.0f);
 
-    Selection pickDeepSkyObject(const UniversalCoord&,
-                                const Vec3f&,
+    Selection pickDeepSkyObject(const UniversalCoord& origin,
+                                const Vec3f& direction,
+                                int   renderFlags,
                                 float faintest,
                                 float tolerance = 0.0f);
 
     Selection find(const std::string& s,
                    Selection* contexts = NULL,
                    int nContexts = 0,
-                   bool i18n = false);
+                   bool i18n = false) const;
     Selection findPath(const std::string& s,
                        Selection* contexts = NULL,
                        int nContexts = 0,
-                       bool i18n = false);
+                       bool i18n = false) const;
     Selection findChildObject(const Selection& sel,
                               const string& name,
                               bool i18n = false) const;
@@ -100,7 +102,8 @@ class Universe
                     float size,
                     Color color,
                     Marker::Symbol symbol,
-                    int priority);
+                    int priority,
+                    string label);
     void unmarkObject(const Selection&, int priority);
     void unmarkAll();
     bool isMarked(const Selection&, int priority) const;
