@@ -14,7 +14,7 @@
 #include <celutil/directory.h>
 #include <celutil/util.h>
 #include <celengine/celestia.h>
-#include <celengine/parser.h>
+#include <celengine/texmanager.h>
 #include "configfile.h"
 
 using namespace std;
@@ -37,7 +37,7 @@ CelestiaConfig* ReadCelestiaConfig(string filename, CelestiaConfig *config)
     ifstream configFile(filename.c_str());
     if (!configFile.good())
     {
-        DPRINTF(0, "Error opening config file.");
+        DPRINTF(0, "Error opening config file '%s'.\n", filename.c_str());
         return config;
     }
 
@@ -67,7 +67,14 @@ CelestiaConfig* ReadCelestiaConfig(string filename, CelestiaConfig *config)
 
     Hash* configParams = configParamsValue->getHash();
 
-    if (config == NULL) config = new CelestiaConfig();
+    if (config == NULL)
+        config = new CelestiaConfig();
+
+#ifdef CELX
+    config->configParams = configParams;  
+    configParams->getString("LuaHook", config->luaHook);    
+    config->luaHook = WordExp(config->luaHook);             
+#endif
 
     config->faintestVisible = 6.0f;
     configParams->getNumber("FaintestVisibleMagnitude", config->faintestVisible);
@@ -100,8 +107,6 @@ CelestiaConfig* ReadCelestiaConfig(string filename, CelestiaConfig *config)
     configParams->getString("TitleFont", config->titleFont);
     configParams->getString("LogoTexture", config->logoTextureFile);
     configParams->getString("Cursor", config->cursor);
-    // configParams->getNumber("LogoWidth", config->logoWidth);
-    // configParams->getNumber("LogoHeight", config->logoHeight);
 
     double aaSamples = 1;
     configParams->getNumber("AntialiasingSamples", aaSamples);
@@ -114,6 +119,8 @@ CelestiaConfig* ReadCelestiaConfig(string filename, CelestiaConfig *config)
     configParams->getNumber("RotateAcceleration", config->rotateAcceleration);
     config->mouseRotationSensitivity = 1.0f;
     configParams->getNumber("MouseRotationSensitivity", config->mouseRotationSensitivity);
+    config->reverseMouseWheel = false;
+    configParams->getBoolean("ReverseMouseWheel", config->reverseMouseWheel);
     configParams->getString("ScriptScreenshotDirectory", config->scriptScreenshotDirectory);
     config->scriptScreenshotDirectory = WordExp(config->scriptScreenshotDirectory);
     config->scriptSystemAccessPolicy = "ask";
@@ -221,35 +228,6 @@ CelestiaConfig* ReadCelestiaConfig(string filename, CelestiaConfig *config)
         }
     }
 
-    Value* labelledStarsVal = configParams->getValue("LabelledStars");
-    if (labelledStarsVal != NULL)
-    {
-        if (labelledStarsVal->getType() != Value::ArrayType)
-        {
-            DPRINTF(0, "%s: LabelledStars must be an array.\n", filename.c_str());
-        }
-        else
-        {
-            Array* labelledStars = labelledStarsVal->getArray();
-            // assert(labelledStars != NULL);
-
-            for (Array::iterator iter = labelledStars->begin(); iter != labelledStars->end(); iter++)
-            {
-                Value* starNameVal = *iter;
-                // assert(starNameVal != NULL);
-                if (starNameVal->getType() == Value::StringType)
-                {
-                    config->labelledStars.insert(config->labelledStars.end(),
-                                                 starNameVal->getString());
-                }
-                else
-                {
-                    DPRINTF(0, "%s: Star name must be a string.\n", filename.c_str());
-                }
-            }
-        }
-    }
-
     Value* ignoreExtVal = configParams->getValue("IgnoreGLExtensions");
     if (ignoreExtVal != NULL)
     {
@@ -273,8 +251,93 @@ CelestiaConfig* ReadCelestiaConfig(string filename, CelestiaConfig *config)
             }
         }
     }
+    
+    Value* starTexValue = configParams->getValue("StarTextures");
+    if (starTexValue != NULL)
+    {
+        if (starTexValue->getType() != Value::HashType)
+        {
+            DPRINTF(0, "%s: StarTextures must be a property list.\n", filename.c_str());
+        }
+        else
+        {
+            Hash* starTexTable = starTexValue->getHash();
+            string starTexNames[StellarClass::Spectral_Count];
+            starTexTable->getString("O", starTexNames[StellarClass::Spectral_O]);
+            starTexTable->getString("B", starTexNames[StellarClass::Spectral_B]);
+            starTexTable->getString("A", starTexNames[StellarClass::Spectral_A]);
+            starTexTable->getString("F", starTexNames[StellarClass::Spectral_F]);
+            starTexTable->getString("G", starTexNames[StellarClass::Spectral_G]);
+            starTexTable->getString("K", starTexNames[StellarClass::Spectral_K]);
+            starTexTable->getString("M", starTexNames[StellarClass::Spectral_M]);
+            starTexTable->getString("R", starTexNames[StellarClass::Spectral_R]);
+            starTexTable->getString("S", starTexNames[StellarClass::Spectral_S]);
+            starTexTable->getString("N", starTexNames[StellarClass::Spectral_N]);
+            starTexTable->getString("WC", starTexNames[StellarClass::Spectral_WC]);
+            starTexTable->getString("WN", starTexNames[StellarClass::Spectral_WN]);
+            starTexTable->getString("Unknown", starTexNames[StellarClass::Spectral_Unknown]);
+            starTexTable->getString("L", starTexNames[StellarClass::Spectral_L]);
+            starTexTable->getString("T", starTexNames[StellarClass::Spectral_T]);
+            starTexTable->getString("C", starTexNames[StellarClass::Spectral_C]);
+            
+            // One texture for all white dwarf types; not sure if this needs to be
+            // changed. White dwarfs vary widely in temperature, so texture choice
+            // should probably be based on that instead of spectral type.
+            starTexTable->getString("WD", starTexNames[StellarClass::Spectral_D]);
+            
+            string neutronStarTexName;
+            if (starTexTable->getString("NeutronStar", neutronStarTexName))
+            {
+                config->starTextures.neutronStarTex.setTexture(neutronStarTexName, "textures");
+            }
+            
+            string defaultTexName;
+            if (starTexTable->getString("Default", defaultTexName))
+            {
+                config->starTextures.defaultTex.setTexture(defaultTexName, "textures");
+            }
+            
+            for (unsigned int i = 0; i < (unsigned int) StellarClass::Spectral_Count; i++)
+            {
+                if (starTexNames[i] != "")
+                {
+                    config->starTextures.starTex[i].setTexture(starTexNames[i], "textures");
+                }
+            }
+        }
+    }
 
-    delete configParamsValue;
+    // TODO: not cleaning up properly here--we're just saving the hash, not the instance of Value
+    config->params = configParams;
+
+#ifndef CELX
+     delete configParamsValue;
+#endif
 
     return config;
 }
+
+
+float
+CelestiaConfig::getFloatValue(const string& name)
+{
+    assert(params != NULL);
+    
+    double x = 0.0;
+    params->getNumber(name, x);
+    
+    return (float) x;
+}
+
+
+const string
+CelestiaConfig::getStringValue(const string& name)
+{
+    assert(params != NULL);
+    
+    Value* v = params->getValue(name);
+    if (v == NULL || v->getType() != Value::StringType)
+        return string("");
+
+    return v->getString();
+}
