@@ -12,16 +12,19 @@
 
 #include <string>
 #include <vector>
+#include <celengine/parser.h>
+#include <celengine/star.h>
 
-struct CelestiaConfig
+
+class CelestiaConfig
 {
+public:
     std::string starDatabaseFile;
     std::string starNamesFile;
     std::vector<std::string> solarSystemFiles;
     std::vector<std::string> starCatalogFiles;
     std::vector<std::string> extrasDirs;
     std::string deepSkyCatalog;
-    std::vector<std::string> labelledStars;
     std::string asterismsFile;
     std::string boundariesFile;
     float faintestVisible;
@@ -37,12 +40,19 @@ struct CelestiaConfig
     std::vector<std::string> ignoreGLExtensions;
     float rotateAcceleration;
     float mouseRotationSensitivity;
+    bool  reverseMouseWheel;
     std::string scriptScreenshotDirectory;
     std::string scriptSystemAccessPolicy;
+#ifdef CELX
+    std::string luaHook;
+    Hash* configParams;
+#endif
 
     std::string HDCrossIndexFile;
     std::string SAOCrossIndexFile;
     std::string GlieseCrossIndexFile;
+    
+    StarDetails::StarTextureSet starTextures;
 
     // Renderer detail options
     unsigned int shadowTextureSize;
@@ -53,9 +63,13 @@ struct CelestiaConfig
     unsigned int aaSamples;
 
     bool hdr;
+    
+    Hash* params;
+    
+    float getFloatValue(const std::string& name);
+    const std::string getStringValue(const std::string& name);
 };
 
-
 CelestiaConfig* ReadCelestiaConfig(std::string filename, CelestiaConfig* config = NULL);
 
 #endif // _CONFIGFILE_H_
