@@ -29,6 +29,10 @@ class MovieCapture
     virtual int getWidth() const = 0;
     virtual int getHeight() const = 0;
     virtual float getFrameRate() const = 0;
+
+    virtual void setAspectRatio(int aspectNumerator, int aspectDenominator) = 0;
+    virtual void setQuality(float) = 0;
+    virtual void recordingStatus(bool started) = 0; /* to update UI recording status indicator */
 };
 
 #endif // _MOVIECAPTURE_H_
