@@ -1,6 +1,6 @@
 // quaternion.h
 // 
-// Copyright (C) 2000, Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2000-2006, Chris Laurel <claurel@shatters.net>
 //
 // Template-ized quaternion math library.
 //
@@ -12,26 +12,11 @@
 #ifndef _QUATERNION_H_
 #define _QUATERNION_H_
 
+#include <limits>
 #include <celmath/mathlib.h>
 #include <celmath/vecmath.h>
 
 
-#if 0
-// Forward declarations needed to make friend templates work . . .
-// Disabled until I can get this to work on every compiler.
-template<class T> class Quaternion;
-
-template<class T> Quaternion<T> operator+(Quaternion<T>, Quaternion<T>);
-template<class T> Quaternion<T> operator-(Quaternion<T>, Quaternion<T>);
-template<class T> Quaternion<T> operator*(Quaternion<T>, Quaternion<T>);
-template<class T> Quaternion<T> operator*(T, Quaternion<T>);
-template<class T> Quaternion<T> operator*(Vector3<T>, Quaternion<T>);
-template<class T> bool operator==(Quaternion<T>, Quaternion<T>);
-template<class T> bool operator!=(Quaternion<T>, Quaternion<T>);
-template<class T> T real(Quaternion<T>);
-template<class T> Vector3<T> imag(Quaternion<T>);
-#endif
-
 template<class T> class Quaternion
 {
 public:
@@ -42,7 +27,7 @@ public:
     inline Quaternion(T, const Vector3<T>&);
     inline Quaternion(T, T, T, T);
 
-    Quaternion(const Matrix3<T>&);
+    inline Quaternion(const Matrix3<T>&);
 
     inline Quaternion& operator+=(Quaternion);
     inline Quaternion& operator-=(Quaternion);
@@ -59,7 +44,10 @@ public:
     Matrix4<T> toMatrix4() const;
     Matrix3<T> toMatrix3() const;
 
-    static Quaternion<T> slerp(Quaternion<T>, Quaternion<T>, T);
+    static Quaternion<T> slerp(const Quaternion<T>&, const Quaternion<T>&, T);
+    static Quaternion<T> vecToVecRotation(const Vector3<T>& v0,
+                                          const Vector3<T>& v1);
+    static Quaternion<T> matrixToQuaternion(const Matrix3<T>& m);
 
     void rotate(Vector3<T> axis, T angle);
     void xrotate(T angle);
@@ -73,41 +61,7 @@ public:
     static Quaternion<T> xrotation(T);
     static Quaternion<T> yrotation(T);
     static Quaternion<T> zrotation(T);
-#if 0
-    // This code is disabled until I can 
-#ifndef BROKEN_FRIEND_TEMPLATES
-    // This just rocks . . .  MSVC 6.0 doesn't parse this correctly,
-    // so template friend functions need to be declared in the standard
-    // conforming way for GNU C and a non-conforming way for MSVC.
-    friend Quaternion<T> operator+ <>(Quaternion<T>, Quaternion<T>);
-    friend Quaternion<T> operator- <>(Quaternion<T>, Quaternion<T>);
-    friend Quaternion<T> operator* <>(Quaternion<T>, Quaternion<T>);
-    friend Quaternion<T> operator* <>(T, Quaternion<T>);
-    friend Quaternion<T> operator* <>(Vector3<T>, Quaternion<T>);
 
-    friend bool operator== <>(Quaternion<T>, Quaternion<T>);
-    friend bool operator!= <>(Quaternion<T>, Quaternion<T>);
-
-    friend T real<>(Quaternion<T>);
-    friend Vector3<T> imag<>(Quaternion<T>);
-#else
-    friend Quaternion<T> operator+(Quaternion<T>, Quaternion<T>);
-    friend Quaternion<T> operator-(Quaternion<T>, Quaternion<T>);
-    friend Quaternion<T> operator*(Quaternion<T>, Quaternion<T>);
-    friend Quaternion<T> operator*(T, Quaternion<T>);
-    friend Quaternion<T> operator*(Vector3<T>, Quaternion<T>);
-
-    friend bool operator==(Quaternion<T>, Quaternion<T>);
-    friend bool operator!=(Quaternion<T>, Quaternion<T>);
-
-    friend T real(Quaternion<T>);
-    friend Vector3<T> imag(Quaternion<T>);
-#endif // BROKEN_FRIEND_TEMPLATES
-
-#endif
-
-    // Until friend templates are working . . .
-    // private:
     T w, x, y, z;
 };
 
@@ -147,12 +101,14 @@ template<class T> Quaternion<T>::Quaternion(T _w, T _x, T _y, T _z) :
 }
 
 // Create a quaternion from a rotation matrix
+// TODO: purge this from code--it is replaced by the matrixToQuaternion()
+// function.
 template<class T> Quaternion<T>::Quaternion(const Matrix3<T>& m)
 {
     T trace = m[0][0] + m[1][1] + m[2][2];
     T root;
 
-    if (trace > 0)
+    if (trace >= (T) -1.0 + 1.0e-4f)
     {
         root = (T) sqrt(trace + 1);
         w = (T) 0.5 * root;
@@ -163,6 +119,7 @@ template<class T> Quaternion<T>::Quaternion(const Matrix3<T>& m)
     }
     else
     {
+        // Identify the largest element of the diagonal
         int i = 0;
         if (m[1][1] > m[i][i])
             i = 1;
@@ -181,7 +138,6 @@ template<class T> Quaternion<T>::Quaternion(const Matrix3<T>& m)
     }
 }
 
-
 template<class T> Quaternion<T>& Quaternion<T>::operator+=(Quaternion<T> a)
 {
     x += a.x; y += a.y; z += a.z; w += a.w;
@@ -574,33 +530,150 @@ template<class T> T dot(Quaternion<T> a, Quaternion<T> b)
 }
 
 
-template<class T> Quaternion<T> Quaternion<T>::slerp(Quaternion<T> q0,
-                                                     Quaternion<T> q1,
+/*! Spherical linear interpolation of two unit quaternions. Designed for
+ *  interpolating rotations, so shortest path between rotations will be
+ *  taken.
+ */
+template<class T> Quaternion<T> Quaternion<T>::slerp(const Quaternion<T>& q0,
+                                                     const Quaternion<T>& q1,
                                                      T t)
 {
-    T c = dot(q0, q1);
+    const double Nlerp_Threshold = 0.99999;
 
+    T cosAngle = dot(q0, q1);
+
+    // Assuming the quaternions representat rotations, ensure that we interpolate
+    // through the shortest path by inverting one of the quaternions if the
+    // angle between them is negative.
+    Quaternion qstart;
+    if (cosAngle < 0)
+    {
+        qstart = -q0;
+        cosAngle  = -cosAngle;
+    }
+    else
+    {
+        qstart = q0;
+    }
+
+    // Avoid precision troubles when we're near the limit of acos range and
+    // perform a linear interpolation followed by a normalize when interpolating
+    // very small angles.
+    if (cosAngle > (T) Nlerp_Threshold)
+    {
+        Quaternion<T> q = (1 - t) * qstart + t * q1;
+        q.normalize();
+        return q;
+    }
+
+    // Below code unnecessary since we've already inverted cosAngle if it's negative.
+    // It will be necessary if we change slerp to not assume that we want the shortest
+    // path between two rotations.
+#if 0
     // Because of potential rounding errors, we must clamp c to the domain of acos.
-    if (c > (T) 1.0)
-        c = (T) 1.0;
-    else if (c < (T) -1.0)
-        c = (T) -1.0;
+    if (cosAngle < (T) -1.0)
+        cosAngle = (T) -1.0;
+#endif
 
-    T angle = (T) acos(c);
+    T angle = (T) acos(cosAngle);
+    T interpolatedAngle = t * angle;
 
-    if (abs(angle) < (T) 1.0e-5)
-        return q0;
+    // qstart and q2 will form an orthonormal basis in the plane of interpolation.
+    Quaternion q2 = q1 - qstart * cosAngle;
+    q2.normalize();
 
+    return qstart * (T) cos(interpolatedAngle) + q2 * (T) sin(interpolatedAngle);
+#if 0
     T s = (T) sin(angle);
     T is = (T) 1.0 / s;
 
     return q0 * ((T) sin((1 - t) * angle) * is) +
            q1 * ((T) sin(t * angle) * is);
+#endif
 }
 
 
-// Assuming that this is a unit quaternion representing an orientation,
-// apply a rotation of angle radians about the specfied axis
+/*! Return a unit quaternion that representing a rotation that will
+ *  rotation v0 to v1 about the axis perpendicular to them both. If the
+ *  vectors point in opposite directions, there is no unique axis and
+ *  (arbitrarily) a rotation about the x axis will be chosen.
+ */
+template<class T> Quaternion<T> Quaternion<T>::vecToVecRotation(const Vector3<T>& v0,
+                                                                const Vector3<T>& v1)
+{
+    // We need sine and cosine of half the angle between v0 and v1, so
+    // compute the vector halfway between v0 and v1. The cross product of
+    // half and v1 gives the imaginary part of the quaternion
+    // (axis * sin(angle/2)), and the dot product of half and v1 gives
+    // the real part.
+    Vector3<T> half = (v0 + v1) * (T) 0.5;
+
+    T hl = half.length();
+    if (hl > (T) 0.0)
+    {
+        half = half / hl; // normalize h
+
+        // The magnitude of rotAxis is the sine of half the angle between
+        // v0 and v1.
+        Vector3<T> rotAxis = half ^ v1;
+        double cosAngle = half * v1;
+        return Quaternion<T>(cosAngle, rotAxis.x, rotAxis.y, rotAxis.z);
+    }
+    else
+    {
+        // The vectors point in exactly opposite directions, so there is
+        // no unique axis of rotation. Rotating v0 180 degrees about any
+        // axis will map it to v1; we'll choose the x-axis.
+        return Quaternion<T>((T) 0.0, (T) 1.0, (T) 0.0, (T) 0.0);
+    }
+}
+
+
+/*! Create a quaternion from a rotation matrix
+ */
+template<class T> Quaternion<T> Quaternion<T>::matrixToQuaternion(const Matrix3<T>& m)
+{
+    Quaternion<T> q;
+    T trace = m[0][0] + m[1][1] + m[2][2];
+    T root;
+    T epsilon = std::numeric_limits<T>::epsilon() * (T) 1e3;
+
+    if (trace >= epsilon - 1)
+    {
+        root = (T) sqrt(trace + 1);
+        q.w = (T) 0.5 * root;
+        root = (T) 0.5 / root;
+        q.x = (m[2][1] - m[1][2]) * root;
+        q.y = (m[0][2] - m[2][0]) * root;
+        q.z = (m[1][0] - m[0][1]) * root;
+    }
+    else
+    {
+        // Set i to the largest element of the diagonal
+        int i = 0;
+        if (m[1][1] > m[i][i])
+            i = 1;
+        if (m[2][2] > m[i][i])
+            i = 2;
+        int j = (i == 2) ? 0 : i + 1;
+        int k = (j == 2) ? 0 : j + 1;
+
+        root = (T) sqrt(m[i][i] - m[j][j] - m[k][k] + 1);
+        T* xyz[3] = { &q.x, &q.y, &q.z };
+        *xyz[i] = (T) 0.5 * root;
+        root = (T) 0.5 / root;
+        q.w = (m[k][j] - m[j][k]) * root;
+        *xyz[j] = (m[j][i] + m[i][j]) * root;
+        *xyz[k] = (m[k][i] + m[i][k]) * root;
+    }
+
+    return q;
+}
+
+
+/*! Assuming that this is a unit quaternion representing an orientation,
+ *  apply a rotation of angle radians about the specfied axis
+ */
 template<class T> void Quaternion<T>::rotate(Vector3<T> axis, T angle)
 {
     Quaternion q;
