@@ -17,6 +17,10 @@
 #include <iostream>
 #include <functional>
 
+// A little trickery to get something like a compile time assert in C++
+#define COMPILE_TIME_ASSERT(pred) \
+    switch(0){case 0: case pred:;}
+
 #ifndef min
 #define min(a, b) ((a) < (b) ? (a) : (b))
 #endif
@@ -26,13 +30,15 @@
 #endif
 				      
 #ifdef _WIN32
-#define _(s) s
+#include "libintl.h"
+#define _(string) gettext (string)
 #else
-#define _(s) gettext(s)
+#define _(string) gettext (string)
 #endif
 
 extern int compareIgnoringCase(const std::string& s1, const std::string& s2);
 extern int compareIgnoringCase(const std::string& s1, const std::string& s2, int n);
+extern std::string LocaleFilename(const std::string & filename);
 
 class CompareIgnoringCasePredicate : public std::binary_function<std::string, std::string, bool>
 {
